package TrEd::EngValLex::LibXMLPropBank;

use strict;
use IO;
use XML::LibXML;
use XML::LibXML::Iterator;


sub parser_start {
  my ($self, $file, $novalidation)=@_;
  my $parser;
  if ($^O eq 'MSWin32' and $XML::LibXML::VERSION lt '1.49') {
    $parser=XML::LibXML->new(
                             ext_ent_handler => sub {
                               my $f=$file;
                               $f=~s!([\\/])[^/\\]+$!$1!;
                               local *F;
                               print STDERR "Trying to open dtd at $f$_[0]\n";
                               open(F,"$f$_[0]") || print STDERR "failed to open dtd\n";
                               my @f=<F>;
                               close F;
                               return join "",@f;
                             }
                            );
  } else {
    $parser=XML::LibXML->new();
  }
  return () unless $parser;
  if (!$novalidation) {
    $parser->validation(1);
    $parser->load_ext_dtd(1);
    $parser->expand_entities(1);
  } else {
    $parser->validation(0);
    $parser->load_ext_dtd(0);
    $parser->expand_entities(0);
  }
  my $doc;
  print STDERR "parsing file $file\n";
  eval {
      $doc=$parser->parse_file($file);
  };
  print STDERR "$@\ndone\n";
  die "$@\n" if $@;
  $doc->indexElements() if ref($doc) and $doc->can('indexElements');
  return ($parser,$doc);
}

sub get_predicate {
  my ($self, $doc)=@_;
  return undef unless $doc;
  my $root = $doc->getDocumentElement;

  return $root->findvalue('/frameset/predicate/@lemma');
}

sub get_rolesets {
  my ($self, $doc)=@_;
  return undef unless $doc;
  my $root = $doc->getDocumentElement;

  my @res;
  for my $roleset ($root->findnodes('/frameset/predicate/roleset')) {
    push @res, {'id'=>$roleset->getAttribute('id'), name=>$roleset->getAttribute('name')};
  }
  return @res;
}

sub get_roles {
  my ($self, $doc, $roleset_id)=@_;
  return undef unless $doc;
  my $root = $doc->getDocumentElement;

  my @res;
  for my $role ($root->findnodes('/frameset/predicate/roleset[@id="'.$roleset_id.'"]/roles/role')) {
    push @res, {'n'=>$role->getAttribute('n'), 'f'=>$role->getAttribute('f'), descr=>$role->getAttribute('descr')};
  }
  return @res;
}

sub get_examples {
  my ($self, $doc, $roleset_id)=@_;
  return undef unless $doc;
  my $root = $doc->getDocumentElement;

  my @res;
  for my $example ($root->findnodes('/frameset/predicate/roleset[@id="'.$roleset_id.'"]/example')) {
    my $text = $example->findnodes('text')->[0]->toString;
    $text=~s/<\/?text>//g;
    $text=~s/\n//g;
    push @res, {'name'=>$example->getAttribute('name'), 'text'=>$text};
  }
  return @res;
}

sub checkArgLink {
  my ($self, $doc, $roleset, $n, $f) = @_;
  return undef unless $doc;
  my $root = $doc->getDocumentElement;

  my @res = $root->findnodes('//roleset[@id="'.$roleset.'"]/roles/role[@n="'.$n.'"]'.($f?'[@f="'.$f.'"]':''));
  
  return @res;
}

1;
