package TrEd::EngValLex::LibXMLVerbNet;

use strict;
use IO;
use XML::LibXML;

#---------------------------------------------------------------
sub new { bless {} }

sub createParser {
  my ($self, $novalidation)=@_;
  my $parser;
  if ($^O eq 'MSWin32' and $XML::LibXML::VERSION lt '1.49') {
    $parser=XML::LibXML->new(
#                             ext_ent_handler => sub {
#                               my $f=$file;
#                               $f=~s!([\\/])[^/\\]+$!$1!;
#                               local *F;
#                               print STDERR "Trying to open dtd at $f$_[0]\n";
#                               open(F,"$f$_[0]") || print STDERR "failed to open dtd\n";
#                               my @f=<F>;
#                               close F;
#                               return join "",@f;
#                             }
                            );
  } else {
    $parser=XML::LibXML->new();
  }
  return () unless $parser;
  if (!$novalidation) {
    $parser->validation(1);
    $parser->load_ext_dtd(1);
    $parser->expand_entities(1);
  } else {
    $parser->validation(0);
    $parser->load_ext_dtd(0);
    $parser->expand_entities(0);
  }
  print STDERR "parser loaded\n";
  return $parser;
}

sub loadVerbNet {
  my ($self, $dir) = @_;
  my $parser = $self->createParser(1);
  my $doc;
  my $verbList;
 
  opendir(DIR, $dir) || warn "can't opendir $dir: $!";
  my @VN_files = grep { /.xml$/ } readdir(DIR);
  closedir DIR;	   
  foreach my $file (@VN_files) {
    chomp $file;
    print STDERR "Parsing $file ";
    eval {
      $doc=$parser->parse_file($dir."/".$file);
    };
    print STDERR "$@\ndone\n";
    
    my $root = $doc->documentElement();
    map {
      my $verb = $_->getAttribute("name");
      my %newnode;
      $newnode{class} = $_->find("../..")->[0];
      my $ID = $_->find("../..")->[0]->getAttribute("ID");
      $newnode{text} = $ID;
      $verbList->{$verb}->{$ID} = \%newnode;
      } $root->findnodes("MEMBERS/MEMBER");
    
    map {
      my $verb = $_->getAttribute("name");
      my %newnode;
      $newnode{class} = $_->find("../../../..")->[0];
      $newnode{subclass} = $_->find("../..")->[0];
      my $ID = $_->find("../..")->[0]->getAttribute("ID");
      $newnode{text} = $ID;
      $verbList->{$verb}->{$ID} = \%newnode;
      } $root->findnodes("SUBCLASSES/VNSUBCLASS/MEMBERS/MEMBER");
  }
  return $verbList;
}
#----------------------------------------------------------------
sub printSELRESTR {
  my ($self, $node) = @_;
  return "" unless $node;
  
  return $node->getAttribute('Value').$node->getAttribute('type');
}
sub printSELRESTRS {
  my ($self, $node) = @_;
  return "" unless $node;
  my $conc = " ";
  my $logic = $node->getAttribute('logic');
  $conc = $logic  if defined($logic);
  my $all = join $conc, (
    map {
      my $res;
      if ($_->nodeName eq 'SELRESTR') {$res=$self->printSELRESTR($_);}
      if ($_->nodeName eq 'SELRESTRS') {$res=$self->printSELRESTRS($_);}
      $_ = $res;
      } ($node->findnodes('SELRESTR|SELRESTRS'))
      );
  

  return "[".$all."]";
}
sub printSyntax {
  my ($self, $node) = @_;
  return "" unless $node;
  
  my $x = join "\t", map {
    my $res = $_->nodeName;
    my $value = $_->getAttribute('value');
    $res .= "(".$value.")"  if defined ($value);
    $_ = $res;
  } ($node->findnodes('*'));
  return $x;
}

sub loadVNClass {
  my ($self, $class) = @_;
  my ($WNSenses, $ThemRoles, $Frames, $VerbsInSameClass);
  foreach my $mem ($class->find('MEMBERS/MEMBER/@wn')->get_nodelist) {
    map {push(@{$VerbsInSameClass}, $_); } split / /, $mem->value;
  }
  foreach my $mem ($class->findnodes('THEMROLES/THEMROLE')) {
    push (@{$ThemRoles}, $mem->getAttribute('type') . $self->printSELRESTRS($mem->findnodes('SELRESTRS')->[0]));
  }
  foreach my $mem ($class->findnodes('FRAMES/FRAME')) {
    my $description = $mem->findnodes('DESCRIPTION')->[0];
    $Frames .= $description->getAttribute('primary'). "\t";
    $Frames .= $description->getAttribute('secondary'). "\n";
    $Frames .= $self->printSyntax($mem->findnodes("SYNTAX")->[0]). "\n";
    map { $Frames .= $_->textContent. "\n"; } $mem->findnodes('EXAMPLES/EXAMPLE');
    $Frames .= "\n";
  }
  
  return ($WNSenses, $ThemRoles, $Frames, $VerbsInSameClass);
}

1;
