#!/usr/bin/perl

package TrEd::EngValLex::VerbNetEditor;

use strict;
use Tk;
use Tk::BrowseEntry;
use Tk::Label;
use Tk::LabFrame;
use Tk::Adjuster;
use Tk::ROText;
use File::Basename;

use TrEd::EngValLex::LibXMLVerbNet;

my $VerbNetPath; # = '/home/semecky/projekty/eng-pdt/eng-vallex/data/verbnet';

#---------------------------------------------------------------
sub new { bless {} }

#---------------------------------------------------------------
sub search_verb {
    my ($self, $verb_par) = @_;
    my $verb = $self->{verb_entry};
    
    if (defined($verb_par)) {
        $verb = $verb_par;
        $verb =~ s/^?//;
        $verb =~ s/\t.*$//;
    }
    return $self->open_verb($verb, 1);
}   
    
#---------------------------------------------------------------
# fills in the form according to given lemma (string representation)
# $do_not_load   ... if set to 1, it does not load the lexicon if it is not loaded
sub open_verb {
    my ($self, $verb, $load) = @_;
    return unless $verb;

    my $verbData = $self->getLemma($verb, $load);
    if (not defined($verbData)) {
        print STDERR "Verb $verb not found in VerbNet\n";
        return;
    }
    
    $self->{verb_entry} = $verb;
    
    $self->{verbData} = $verbData;
    $self->{verb} = $verb;
    my @list;
    foreach my $mem (keys(%{$self->{verbData}})) {
        push @list, $mem;
        }
    $self->{class_list}->configure(-options=>\@list);
    $self->changeClass;
}

#---------------------------------------------------------------
# returns the XML representations of a given lemma
sub getLemma {
    my ($self, $lemma, $load) = @_;
    if (not defined($self->{verb_list})) {
        warn ("Load: ".$load);
        if ($load) {
            $self->loadVerbNetFiles();
            warn('VerbNet could not be loaded') if not defined($self->{verb_list});
        } else {
            return;
        }   
    }
    return $self->{verb_list}->{$lemma};
}
    
#---------------------------------------------------------------
sub addToForm {
    my ($self, $class, $subclass) = @_;
    my ($WNSenses1, $ThemRoles1, $Frames1, $VerbsInSameClass1);
    ($WNSenses1, $ThemRoles1, $Frames1, $VerbsInSameClass1) =
            TrEd::EngValLex::LibXMLVerbNet->loadVNClass($class) if defined ($class);
    my ($WNSenses2, $ThemRoles2, $Frames2, $VerbsInSameClass2);
    ($WNSenses2, $ThemRoles2, $Frames2, $VerbsInSameClass2) =
            TrEd::EngValLex::LibXMLVerbNet->loadVNClass($subclass) if defined ($subclass);

    #$self->{wordnet_senses}->configure(-text=>$WNSenses1+", "+$WNSenses2);
    
    $self->{thematic_roles}->delete(0, 'end');
    map { $self->{thematic_roles}->insert('end', $_); } @{$ThemRoles1};
    map { $self->{thematic_roles}->insert('end', $_); } @{$ThemRoles2};
    
    $self->{frames_content}->delete(0.1, 'end');
    $self->{frames_content}->insert('end', $Frames1);
    $self->{frames_content}->insert('end', $Frames2);

    # using only class xor subclass co-member
    $self->{verbs_in_the_same_class}->delete(0, 'end');
    if (!defined($subclass)) {
       map { $_ =~ s/%/\t/; $self->{verbs_in_the_same_class}->insert('end', $_) } @{$VerbsInSameClass1};
    } else {
       map { $_ =~ s/%/\t/; $self->{verbs_in_the_same_class}->insert('end', $_) } @{$VerbsInSameClass2};
    }
}
    
#---------------------------------------------------------------
sub loadVerbNetFiles {
    my $self = shift;
    return unless $self;
    
    eval { require TrEd::EngValLex::LibXMLVerbNet; };
    die $@ if $@;
    $self->{verb_list} = TrEd::EngValLex::LibXMLVerbNet->loadVerbNet($VerbNetPath);
    # TODO
    return;
}

#---------------------------------------------------------------
sub changeClass {
    my $self = shift;
    my $class = $self->{class};
    return unless $class;

    $self->addToForm($self->{verbData}->{$self->{class}}->{class}, $self->{verbData}->{$self->{class}}->{subclass}, $self->{verb});
    #TODO
}

# create the VerbNet widget
#---------------------------------------------------------------
sub createVerbNetWidget {
    my ($self, $top, $conf) = @_;
    return unless $top;
    
    if (defined($conf)) {
        if (defined($conf->{VerbNet_dir})) {
            $VerbNetPath = $conf->{VerbNet_dir};
        }
    }
    
    # Overall layout: narrow top frame + two large frames bellow it
    my $top_frame = $top->LabFrame(-label=>'Verb + Class');
    my $left_frame = $top->Frame();
    my $right_frame = $top->Frame();
    my $adj1 = $top->Adjuster(-widget=>$left_frame);
    
    $top_frame->pack(-side=>'top', -fill=>'x', -expand=>'0');
    $left_frame->pack(-side=>'left', -fill=>'both', -expand=>'1');
    $adj1->packAfter($left_frame, -side => 'left');
    $right_frame->pack(-side=>'right', -fill=>'both', -expand=>'1');
    
    # top_frame
    $top_frame->Label(-text=>'Search verb:')->pack(-side=>'left');
    my $search_entry = $top_frame->Entry(-textvariable=>\$self->{verb_entry})->pack(-side=>'left');
    $search_entry->bind('<Return>', sub { $self->search_verb(); } );
    $top_frame->Button(-text=>'Search verb', -command => sub { $self->search_verb; } )->pack(-side=>'left');
    $top_frame->Label(-text=>'Class:')->pack(-side=>'left');
    $self->{class_list} = $top_frame->Optionmenu(-options => [()], -variable => \$self->{class}, -command => sub { $self->changeClass })->pack(-side=>'left');
    
    # left_frame
    #my $wordnet_senses = $left_frame->LabFrame(-label=>'WordNet senses')->pack(qw/-side top -fill both -expand 1/);
    my $thematic_roles = $left_frame->LabFrame(-label=>'Thematic roles')->pack(qw/-side top -fill both -expand 1/);
    #my $adj2 = $left_frame->Adjuster(-widget=>$thematic_roles)->pack(-side=>'top');
    my $verbs_in_the_same_class = $left_frame->LabFrame(-label=>'Verbs in the same class')->pack(qw/-side top -fill both -expand 1/);
    

    #$self->{wordnet_senses} = $wordnet_senses->Label()->pack(qw/-fill both -expand 1/);
    $self->{thematic_roles} = $thematic_roles->Scrolled(qw/Listbox -scrollbars osoe -height 4/)->pack(qw/-side top -fill both -expand 1/);
    
    $self->{verbs_in_the_same_class} = $verbs_in_the_same_class->Scrolled(qw/Listbox -scrollbars osoe -height 12/)->pack(qw/-side top -fill both -expand 1/);
                                                                #Label()->pack(qw/-fill both -expand 1/);
    $self->{verbs_in_the_same_class}->bind('<Double-ButtonPress-1>', sub { $self->search_verb($self->{verbs_in_the_same_class}->get($self->{verbs_in_the_same_class}->curselection->[0])); } );
    
    # right_frame
    my $frames = $right_frame->LabFrame(-label=>'Frames')->pack(qw/-side top -fill both -expand 1/);
    $self->{frames_content} = $frames->Scrolled(qw/ROText -scrollbars osoe -wrap word -height 10/)->pack(qw/-fill both -expand 1/);
    
    $search_entry->focus();

    # Load data
    #$self->loadVerbNetFiles();
}


# to be called when run stand-alone
sub start {
    my $top = MainWindow->new(-title => 'VerbNet viewer');
    $top->geometry('400x400');
    my $vne = TrEd::EngValLex::VerbNetEditor::new();
    $vne->createVerbNetWidget($top);
    MainLoop;
}

#start();
1;
