#-*- cperl -*-

package PML_Cz_T_Coref;

#binding-context PML_Cz_T_Coref

=pod

=head1 PML_Cz_T_Coref

PML_Cz_T_Coref.mak - Miscellaneous macros for annotating the
coreference relations on the tectogrammatic layer of Prague
Czech-English Dependency Treebank.

=over 4

=cut

BEGIN { import PML_T; }

#include <contrib/support/unbind_edit.inc>

#key-binding-adopt PML_T_Edit
#menu-binding-adopt PML_T_Edit

#unbind-key G
#unbind-key N
#unbind-key B
#unbind-key t
#unbind-key p
#unbind-key g
#unbind-key m
#unbind-key r
#unbind-key Insert
#unbind-key Delete
#unbind-key Ctrl+Delete
#unbind-key Ctrl+A
#unbind-key f
#unbind-key l
#unbind-key +
#unbind-key Ctrl+Left
#unbind-key Ctrl+Right
#unbind-key Alt+Left
#unbind-key Alt+Right
#unbind-key Ctrl+!
#unbind-key Ctrl+?
#unbind-key !
#unbind-key ?
#unbind-key Ctrl+Return

#remove-menu Change is_parenthesis
#remove-menu Change is_generated
#remove-menu Change is_member
#remove-menu Delete Node
#remove-menu Delete Subtree
#remove-menu Display corresponding analytical tree
#remove-menu Edit Functor
#remove-menu Edit t_lemma
#remove-menu Select and assign valency frame
#remove-menu Show valency frames and highlight assigned
#remove-menu Edit Node Type
#remove-menu Insert New Node
#remove-menu Edit TFA
#remove-menu Edit Grammatemes
#remove-menu Mark for reference changes and enter A-layer
#remove-menu Move node to the left
#remove-menu Move node to the right
#remove-menu Move subtree to the left
#remove-menu Move subtree to the right

# 01-03-2010
#unbind key Alt+j
#remove-menu Jump to complement coreference antecedent


# For admins: do not disable other contexts
#ifndef pcedt-coref-admin
#ifdef TRED
register_start_hook(sub {
  if(ref $grp->{framegroup}->{UserMenu}){ # skip if called from print.btred
    my $usermenu = $grp->{framegroup}->{UserMenu};
    $usermenu = $usermenu->menu if $usermenu->isa('Tk::Menu::Cascade');
    for my $idx (reverse(0..$usermenu->index('last'))) {
      next if eval { $usermenu->entrycget($idx,'-label') =~ /PML_.*_Anot/ } ;
      $usermenu->delete($idx);
    }
  }
}); #start_hook
#endif TRED
#endif pcedt-coref-admin

sub CreateStylesheets{
  SetStylesheetPatterns(<<'EOF','PML_T_Coref',1);
node:<? if($${is_deleted}){'#{red}'}
        elsif($${is_parenthesis}){'#{customparenthesis}'}
  ?><? $${nodetype}eq'root' ? '${id}' : '${t_lemma}' ?><?
  $${annot_comment}?
  join('',' ',map{if(/valency/){'#{green}!'}elsif(/M-l/){'#{#506aff}M'}
    elsif(/T-lemma$/){'#{#0000e7}L'}elsif(/under/){'#{#0000e7}!'}
    elsif(/Typo/){'#{#506aff}!'}elsif(/Sent/){'#{brown}s'}
    elsif(/Word/){'#{brown}w'}elsif(/M-t/){'#{#506aff}t'}elsif(/Sec/){'#{red}#'}
    elsif(/Other|Functor|Structure|CorefStructure/){'#{red}?'}elsif(/Not|Re/){'#{red}X'}
    elsif(/Vallex/){'#{green}?'}
    elsif(/Coref/){'#{red}C'}
    else{'#{red}!'}
  }map{$_->{type}}ListV($this->{annot_comment}))
  :''?><? '#{customdetail}"' if $${is_dsp_root}
  ?><? '#{customdetail}.${sentmod}'if$${sentmod}
  ?><? '#{customcoref}'.$PML_T::coreflemmas_hash{$${id}}
    if $PML_T::coreflemmas_hash{$${id}}ne'' ?>

full:1

node:<?
  ($${nodetype} eq 'root' ? '#{customnodetype}${nodetype}' :
  '#{customfunc}${functor}').
  "#{customsubfunc}".($${subfunctor}?".\${subfunctor}":'').($${is_state}?".\${is_state=state}":'') ?><? '#{customcoappa}_${is_member=M}'if$${is_member} ?><? '#{customparenthesis}_${is_parenthesis=P}' if$${is_parenthesis} ?>

node:<? my $line;
        if(! $PML_T::showANodes){
          $line=(($this->attr('a/lex.rf')
            ? ('#{darkgreen}'
              .(PML_T::GetANodeByID($this->attr('a/lex.rf'))->attr('m/form')))
            : ''
          ).($this->attr('a/aux.rf')
            ? ('#{darkorange}'
              .join(' ','',map{
                  PML_T::GetANodeByID($_)->attr('m/form')
                }ListV($this->attr('a/aux.rf'))
              ))
            : ''
          ));
        }else{
          $line=(($${nodetype} ne 'complex' and $${nodetype} ne 'root')
            ? '#{customnodetype}${nodetype}'
            : '');
          $line.='#{customcomplex}${gram/sempos}';
        }
        return $line;
?>

style:#{Node-width:7}#{Node-height:7}#{Node-currentwidth:9}#{Node-currentheight:9}

style:<? '#{Node-shape:'.($this->{is_generated}?'rectangle':'oval').'}'?>

style:<? exists $PML_T::show{$${id}} ?'#{Node-addwidth:10}#{Node-addheight:10}':''?>

style:<?
  if(not ($${is_member}) and (
     ($this->{functor}=~/^(?:PAR|PARTL|VOCAT|RHEM|CM|FPHR|PREC)$/) or
     ($this->parent and $this->parent->{nodetype}eq'root'))) {
     '#{Line-width:1}#{Line-dash:2,4}#{Line-fill:'.CustomColor('line_normal').'}'
  } elsif ($${is_member}) {
    if (PML_T::IsCoord($this)and PML_T::IsCoord($this->parent)) {
      '#{Line-width:1}#{Line-fill:'.CustomColor('line_member').'}'
    } elsif ($this->parent and PML_T::IsCoord($this->parent)) {
      '#{Line-coords:n,n,(n+p)/2,(n+p)/2&(n+p)/2,(n+p)/2,p,p}#{Line-width:3&1}#{Line-fill:'.
       CustomColor('line_normal').'&'.CustomColor('line_member').'}'
    } else {
      '#{Line-fill:'.CustomColor('error').'}'
    }
  } elsif ($this->parent and PML_T::IsCoord($this->parent)) {
    '#{Line-width:1}#{Line-fill:'.CustomColor('line_comm').'}'
  } elsif (PML_T::IsCoord($this)) {
    '#{Line-coords:n,n,(n+p)/2,(n+p)/2&(n+p)/2,(n+p)/2,p,p}#{Line-width:1&3}#{Line-fill:'.
    CustomColor('line_member').'&'.CustomColor('line_normal').'}'
  } else {
    '#{Line-width:2}#{Line-fill:'.CustomColor('line_normal').'}'
  }
?>

style:<?
  if($${is_unchecked}){
    my $n = exists $PML_T::show{$${id}} ? '9' : '5' ;
    '#{Oval-fill:#0000a0}#{Node-addwidth:'.$n.'}#{Node-addheight:'.$n.'}'
  }elsif($${is_exception}){
    my $n = exists $PML_T::show{$${id}} ? '9' : '5' ;
    '#{Oval-fill:#ff7400}#{Node-addwidth:'.$n.'}#{Node-addheight:'.$n.'}'
  }elsif ($${tfa}=~/^[tfc]$/) {
    '#{Oval-fill:'.CustomColor('tfa_'.$${tfa}).'}${tfa}.'
  } else {
    '#{Oval-fill:'.CustomColor('tfa_no').'}'
  }
?>#{CurrentOval-width:3}#{CurrentOval-outline:<? CustomColor('current') ?>}

hint:<?
   my @hintlines;
   if (ref($this->{gram})) {
     foreach my $gram (sort keys %{$this->{gram}}) {
       push @hintlines, "gram/".$gram." : ".$this->{gram}->{$gram} if $this->{gram}->{$gram}
     }
   }
   push@hintlines, "is_dsp_root : 1" if $${is_dsp_root};
   push@hintlines, "is_name_of_person : 1" if $${is_name_of_person};
   push@hintlines, "quot : ". join(",",map{$_->{type}}ListV($this->{quot})) if $${quot};
   push @hintlines,map{'! '.$_->{type}.':'.$_->{text}}ListV($this->{annot_comment});
   join"\n", @hintlines
?>
EOF
}#CreateStylesheets

sub get_value_line_hook {
  return &PML_Cz_T_Anot::get_value_line_hook;
} # get_value_line_hook

sub switch_context_hook {
  CreateStylesheets();
  SetCurrentStylesheet('PML_T_Coref');
  Redraw() if GUI();
  my ($prefix,$file)=FindVallex();
  ValLex::GUI::Init({-vallex_file => $file});
}

sub enable_edit_node_hook { 'stop' }

sub enable_attr_hook { 'stop' }

sub node_release_hook{
  return 'stop' if $_[2] =~ /^(?:0|Alt)$/; # disable rehanging nodes and COMPL arrows
  return &PML_Cz_T_Anot::node_release_hook;
}#node_release_hook

sub get_status_line_hook {
  &PML_T_Edit::get_status_line_hook;
}#get_status_line_hook

sub AddComment {
  my $type = $_[0];
  if(defined $this->{annot_comment}){
    my @comment = grep {$_->{type} eq $type} $this->{annot_comment}->values;
    return if $#comment > -1;
  }
  my $text = QueryString("$type Comment text",'Text:','');
  return unless defined $text and length $text;
  my %comment = (type=>$type,
                 text=>$text);
  AddToList($this,'annot_comment',\%comment);
}#AddComment


sub EditComment{
  ChangingFile(0);
  return unless defined $this->{annot_comment};
  my $type = $_[0];
  my @comment = grep {$_->{type} eq $type} $this->{annot_comment}->values;
  return if $#comment < 0;
  my $text = QueryString("$type Comment text",'Text:',$comment[0]->{text});
  if(defined $text and $text ne $comment[0]->{text}){
    $comment[0]->{text} = $text;
    ChangingFile(1);
    $this->{annot_comment}->delete_value($comment[0]) if $text eq '';
  }
}#EditComment


sub AddBen {
  $this=NewNode($this);
  $this->{functor}='BEN'; #  $this->{subfunctor}='nr';
  $this->{t_lemma}='#Benef';
  $this->{nodetype}='qcomplex';
  $this->{is_generated}=1;
  $this->{is_added}=1;
}#AddBen

sub Check {
  ChangingFile(0);
  if($this->{t_lemma} and $this->{t_lemma} ne '#NewNode'
     and $this->{is_unchecked}
     and not ($this->{t_lemma} eq '#Gen'
       and ($this->{'coref_gram.rf'} or $this->{'coref_text.rf'} or $this->{'coref_special'})
     )
     and not ($this->{t_lemma} eq '#Cor' and $this->{'coref_text.rf'})
  ){
    delete $this->{is_unchecked};
    ChangingFile(1);
  }
}

my %filelist_nodes;
sub ToggleException {
  if(not %filelist_nodes){
    my $long = "";
    map {
      m/.*#(.*)/; 
      $filelist_nodes{$1} = 1;
    } @{$win->{currentFilelist}{list}};
  }
  ChangingFile(0);
  if($filelist_nodes{$this->{id}}){#/*node is in filelist*/
    $this->{is_exception} = not $this->{is_exception};
    ChangingFile(1);
  }
}

sub Next_Unchecked {
  ChangingFile(0);
  my $node = $this;
  until($this->{is_unchecked} or $this->following == $node){
    if($this->following){
      $this = $this->following
    }else{
      NextTree() or TredMacro::GotoTree(1);
    }
  }
  unless ($this->{is_unchecked}){
    $this = $node;
    questionQuery('Not found',
                  'No more nodes in the current file.',
                  'OK');
  }
} # Next_Unchecked


sub Change_tlemma {
  ChangingFile(0);
  if($this and $this->{t_lemma} and $this->{t_lemma} =~ /^#(?:Q?Cor|NewNode|Unsp|PersPron|Gen|Rcp)$/){
    $this->{t_lemma} = "#$_[0]";
    $this->{functor} = 'BEN' if $_[0] eq 'Benef';
    ChangingFile(1);
  }
} # Change_tlemma


foreach my $tlemma (qw/Cor Unsp PersPron Gen Rcp/){
  Bind(sub{Change_tlemma($tlemma)} => {key =>lc(substr($tlemma,0,1)),
                                       menu => "Change t_lemma to #$tlemma"});
}

sub Change_functor {
  ChangingFile(0);
  if($this->{t_lemma}
     and $this->{t_lemma} =~ /^#(?:Q?Cor|NewNode|Unsp|PersPron|Gen)$/){
    $this->{functor} = $_[0];
    ChangingFile(1);
  }
} # Change_functor

sub CorefDeleteNode {
  ChangingFile(0);
  if($this->{t_lemma}
     and $this->{t_lemma} =~ /^#(?:Q?Cor|NewNode|Unsp|PersPron|Gen|Benef)$/){
    DeleteLeafNode($this) and ChangingFile(1);
  }
}#CorefDeleteNode

sub CorefDeleteAdded {
  ChangingFile(0);
  if($this->{is_added}){
    DeleteLeafNode($this) and ChangingFile(1);
  }
}#CorefDeleteAdded

sub CorefDeleteOrMark {
  ChangingFile(0);
  if($this->{is_added}){
	CorefDeleteAdded();
  } else {
	ToggleDeletionMark();
  }
}#CorefDeleteOrMark

sub ToggleDeletionMark {
  ChangingFile(0);
  if($this->{t_lemma}
     and $this->{t_lemma} =~ /^#(?:Q?Cor|NewNode|Unsp|PersPron|Gen|Benef)$/){
    if($this->{is_deleted}){
      delete $this->{is_deleted};
    }
    else{
      $this->{is_deleted} = 1;
    }
    ChangingFile(1);
  }
}#ToggleDeletionMark


# foreach my $actant (qw/ACT PAT ADDR EFF ORIG/){
#   Bind(sub{Change_functor($actant)}
#        => {key => $actant eq 'ADDR' ? 'D' : substr($actant,0,1),
#            menu => "Change functor to $actant"});
# }

#bind AddBen to B menu Add #Benef.BEN
#bind CorefDeleteOrMark to Delete menu Delete added node / toggle deletion proposal for original node
#bind Check to o menu Check
#bind ToggleException to P menu Toggle Check Exception Mark
#bind Next_Unchecked to n menu Go to next unchecked node


Bind(sub{AddComment('Coref')} =>  {key => '!', menu => "Add Annotators' comment"});
Bind(sub{EditComment('Coref')} =>  {key => '?', menu => "Edit Annotators' comment"});
Bind(sub{AddComment('CorefStructure')} => {key => 'Ctrl+!', menu => "Add Structure comment"});
Bind(sub{EditComment('CorefStructure')} => {key => 'Ctrl+?', menu => "Edit Structure comment"});

# remove all arrows
sub Remove_all_coref {
  my $changing = 0;
  foreach my $type( ('gram.rf','text.rf','special') ){
    if($this->{"coref_$type"}){
      delete $this->{"coref_$type"};
      $changing = 1;
    }
  }
  ChangingFile($changing);
} #Remove_all_coref

# same as TextArowToRemembered from pdt20/contrib/pml/PML_T_Edit.inc
sub GramArowToRemembered{
  ChangingFile(0);
  if($PML_T_Edit::remember and $PML_T_Edit::remember->parent and $this->parent){
    PML_T_Edit::AddCoref($this,$PML_T_Edit::remember,'coref_gram.rf');
    ChangingFile(1);
  }
}#GramArowToRemembered

#bind Remove_all_coref to Ctrl+x menu Clear all coreference attributes
#bind GramArowToRemembered to Shift+space menu Make Grammatical Coreference Arrow to Remembered Node
#bind PML_T_Edit->ForgetRemembered to x

# ??? after_edit_attr_hook

=back

=cut

1;
