# -*- cperl -*-

BEGIN
{
	import PML_A_Edit;
}

=pod

=head1 PML_En_A

PML_En_A.mak - Miscellaneous macros for the analytic layer of Prague
Dependency Treebank for English PennTreebank files

=over 4

=cut

#ifdef TRED


=item detect()

Returns true if the current file is an English PDT a-file

=cut

sub detect {
  return (PML::SchemaDescription() =~ m{^English PDT} and PML::SchemaName() eq 'adata') ? 1 : 0;
}

sub allow_switch_context_hook {
  return 'stop' unless detect();
}

sub TectogrammaticalTree {
  return unless SchemaName() =~ /^adata$/;
  return unless SwitchToTFile();
  my $arf = $PML::arf;
  my $fsfile = $grp->{FSFile};
  my $id = $root->{id};
  my $this_id = $this->{id};
  my $desired_context = $PML_En_T::last_context ? $PML_En_T::last_context : 'PML_En_T';
  SwitchContext($desired_context);
  if(ref $arf){
    $this=$grp->{currentNode};
    my($node,$treeno)=SearchForNodeById($arf->{id});
    TredMacro::GotoTree($treeno);
    $this=$node;
  }else{
    #find current tree and new $this
    my $trees = $fsfile->treeList;
    # low-level stuff here
    my $treeNo = $fsfile->currentTreeNo+0;
  TREE: for (my $i=0;$i<=$#$trees;$i++) {
      foreach my $a_rf (PML_T::GetANodeIDs($trees->[$i])) {
        $a_rf =~ s/^.*\#//;
        if ($a_rf eq $id) {
          $treeNo = $i;
          last TREE;
        }
      }
    }
    TredMacro::GotoTree($treeNo+1); # changes $root
    my $node=$root;
    while ($node) {
      if (first { $_ eq $this_id } PML_T::GetANodeIDs($node)) {
        $this = $node;
        last;
      }
    } continue {
      $node = $node->following;
    };
  }
  ChangingFile(0);
  1;
}

sub SwitchToTFile {
  my $fsfile = $grp->{FSFile};
  return 0 unless $fsfile or SchemaName() !~ /^adata$/;
  my $tr_fs = $fsfile->appData('tdata');
  if (!$tr_fs)
  {
    $tr_fs = HackToTFile();
  }
  return 0 unless ref($tr_fs);
  $grp->{FSFile} = $tr_fs;
  return 1;
}

sub HackToTFile
{
  my $file_name = FileName();
  $file_name =~ s/\.a.gz$/\.t.gz/;
  my ($req_fs,$status2) = main::openFile($grp,$file_name,-preload => 1);
  return $req_fs;
}

=item PhrasalTree()

This function is only available in TrEd (i.e. in GUI). After a
previous call to C<AnalyticalTree>, it switches current view back to
a tectogrammatical tree which refers to the current analytical tree.

=cut

sub PhrasalTree {
  return unless SchemaName() eq 'adata';
  return unless SwitchToPFile();
  my $a_root = $root;
  my $a_node = $this;
  SwitchContext('PML_En_P');
  my $fsfile = $grp->{FSFile};
  #find current tree and new $this
  my $trees = $fsfile->treeList;
  my $treeNo = $fsfile->currentTreeNo+0;
TREE: for (my $i=0;$i<=$#$trees;$i++) {
   foreach my $p_rf (GetPNodeIDs($a_root)) {
     $p_rf =~ s/^.*?#//;
     if ($trees->[$i]->{id} eq $p_rf) {
       $treeNo = $i;
       last TREE;
     }
   }
 }
  TredMacro::GotoTree($treeNo+1); # changes $root and $this
 my $p_ids = GetNodeHash($fsfile);
 my $first =
   first {
     ref($p_ids->{$_}) and ($p_ids->{$_}->root == $root) ? 1 : 0
   } GetPNodeIDs($a_node);
 $this = $p_ids->{$first};
  ChangingFile(0);
}

sub SwitchToPFile {
  my $fsfile = $grp->{FSFile};
  return 0 unless SchemaName() eq 'adata';
  my $pr_fs = PFile($fsfile);
  return 0 unless $pr_fs;
  # remember the file we came from:
  $pr_fs->changeAppData('adata',$fsfile);
  $grp->{FSFile} = $pr_fs;
  return 1;
}

#endif TRED

sub PFile {
  my $fsfile = $_[0] || $grp->{FSFile};
  return undef unless ref($fsfile->metaData('refnames')) and ref($fsfile->appData('ref'));
  if (defined $fsfile->metaData('refnames')->{pdata}) {
     my $refid = $fsfile->metaData('refnames')->{pdata};
     return $fsfile->appData('ref')->{$refid};
  }
  else {
     my $adata_refid = $fsfile->metaData('refnames')->{adata};
     my $adatafile = $fsfile->appData('ref')->{$adata_refid};
     my $refid = $adatafile->metaData('refnames')->{pdata};
     return $adatafile->appData('ref')->{$refid};
  }
}


=item GetPNodes($node?)

Returns a list of phrasal nodes referenced from a given
analytical node. If no node is given, the function applies to
C<$this>.

=cut

sub GetPNodes {
  shift unless ref($_[0]);
  my $node = $_[0];
  return grep defined, map { GetPNodesHash()->{$_} } &GetPNodeIDs($node);
}

=item GetPNodesHash()

Return a reference to a hash indexing phrasal nodes of the
phrasal file associated with the current analytical file. If
such a hash was not yet created, it is created upon the first call to
this function (or other functions calling it, such as C<GetPNodes>.

=cut

sub GetPNodesHash {
  shift unless ref($_[0]);
  my $fsfile = $_[0] || $grp->{FSFile};
  return {} unless ref($fsfile);
  my $p_fs;
  if ($fsfile->appData('struct') eq 'pdata') {
    $p_fs = $fsfile;
  } else {
    $p_fs = PFile($fsfile);
    return {} unless ref($p_fs);
  }
  return GetNodeHash($p_fs);
}



=item GetPNodeIDs($node?)

Returns a list of IDs of phrasal nodes referenced from a given
analytical node. If no node is given, the function applies to
C<$this>.

=cut

sub GetPNodeIDs {
  return map { my $id=$_; $id=~s/^.*?#//; $id } &GetPNodeREFs;
}

=item GetPNodeREFs($node?)

Returns a list of PMLREFs of phrasal nodes referenced from a given
analytical node. If no node is given, the function applies to
C<$this>. This function is similar to C<GetPNodeIDs()> but it doesn't
strip the file-ref part of the reference.

=cut

sub GetPNodeREFs {
  my $node = $_[0] || $this;
  return $node->{'ptree.rf'} if $node->{'ptree.rf'} ne "";
  return () unless ref($node->{'p'});
  my $lex = $node->{'p'}{'terminal.rf'};
  return (($lex ne "" ? $lex : ()), ListV($node->{'p'}{'nonterminals.rf'}));
}

=item GetPNodeREFs($node?)

Returns reference to the terminal node corresponding to the given node.If no node is given, the function applies to
C<$this>.

=cut

sub GetPTerminalREF {
  my $node = $_[0] || $this;
  return undef if $node->{'ptree.rf'} ne "";
  return undef unless ref($node->{'p'});
  return $node->{'p'}{'terminal.rf'};
}

sub switch_context_hook {
  my $cur_stylesheet = GetCurrentStylesheet();
  if ($cur_stylesheet ne 'PML_En_A'){
    SetCurrentStylesheet('PML_En_A');
    Redraw() if GUI();
  }
  return;
}

sub enable_attr_hook
{
	my $attr = shift;
	if ($attr =~ m/^(m\/(?:tag|form|lemma)|id)$/) { return 'stop'; }
	else { return 1; }
}

#include <contrib/support/arrows.inc>

sub root_style_hook {
  my ($node,$styles)=@_;
  %PML_En_A::coreflemmas_hash=();
  %PML_En_A::displayed_nodes;
  my @displayed = GetDisplayedNodes();
  @PML_En_A::displayed_nodes{ @displayed } = ( 0..$#displayed );
  DrawArrows_init();
  return;
}

sub after_redraw_hook {
  %PML_En_A::coreflemmas_hash=();
  %PML_En_A::displayed_nodes=();
  DrawArrows_cleanup();
  return;
}

sub node_style_hook {
  my ($node,$styles)=@_;
  if  ($node->{'coref_text.rf'}) {
    my @text = grep {$_ ne "" } ListV($node->{'coref_text.rf'});
	DrawCorefArrows($node,$styles,undef, [@text], [(map 'textual',@text)]);
  }
  return 1;
}

sub DrawCorefArrows {
  my ($node,$styles,$line,$corefs,$cortypes)=@_;

  # corefs is an array-ref of node IDs
  # coreftypes is an array-ref of cortypes strings or
  # of hashes { -cortype => 'cortype', ...edge_options... }
  # specifying a cortype and custom edge drawing options passed to
  # DrawArrows from <contrib/support/arrows.inc>

  my ($rotate_prv_snt,$rotate_nxt_snt,$rotate_dfr_doc)=(0,0,0);
  my @edges;
  my %common = (
    -arrow => 'last',
    -smooth => 1,
    -arrowshape => '16,18,3',
    -width => 1,
    -dash => ''
  );
  my @displayed = GetDisplayedNodes();
  @PML_En_A::displayed_nodes{ @displayed } = ( 0..$#displayed );

  foreach my $coref (@$corefs) {
    my $cortype=shift @$cortypes;
    my($refed,$treeNo)=SearchForNodeById($coref);
    my %edge = (-tag => $cortype);
    if ($refed) {
      my $orientation=
	exists($PML_En_A::displayed_nodes{$refed}) ? 0 : $treeNo-CurrentTreeNumber()-1;
      $PML_En_A::coreflemmas_hash{$node->{id}}.=' '.$refed->attr('m/lemma') if $orientation;
      if ($orientation == 0) {
	$edge{-target} = $refed;
	$edge{-fill}=CustomColor('arrow_'.$cortype);
	$edge{-dash}='';
      } elsif ($orientation<0) {
	print STDERR "ref-arrows: Preceding sentence\n"if $main::macroDebug;
	$edge{-fill}=CustomColor('arrow_'.$cortype);
	$edge{-deltax}=-30;
	$edge{-deltay}=10*($rotate_prv_snt++);
      }else{ #right
	print STDERR "ref-arrows: Following sentence\n" if $main::macroDebug;
	$edge{-fill}=CustomColor('arrow_'.$cortype);
	$edge{-deltax}=+30;
	$edge{-deltay}=10*($rotate_nxt_snt++);
      }
    } else {
      print STDERR "ref-arrows: Not found!\n" if $main::macroDebug;
      $edge{-fill}=CustomColor('error');
      $edge{-deltax}=10*($rotate_dfr_doc++);
      $edge{-deltay}=-25;
    }
    push @edges,{%edge};
  }
  DrawArrows($node,$styles,\@edges,\%common);
}

sub ANodeToALexRf {
  my ($a_node,$t_node,$t_file)=@_;
  return unless ref($t_node) && ref($a_node) && ref($t_file);
  my $refid = $t_file->metaData('refnames')->{adata};
  $t_node->set_attr('a/lex.rf',$refid."#".$a_node->{id});
  $t_node->set_attr('a/aux.rf',List(grep{ $_ ne $refid."#".$a_node->{id} }
    uniq(ListV($t_node->attr('a/aux.rf')))));
  my$lemma=$a_node->attr('m/lemma');
  my%specialEntity;
  %specialEntity=('.' => 'Period',
                  ',' => 'Comma',
                  '&amp;' => 'Amp',
		  '&' => 'Amp',
		  '-' => 'Dash',
		  '--' => 'Dash',
		  '/' => 'Slash',
		  '(' => 'Bracket',
		  ')' => 'Bracket',
		  ';' => 'Semicolon',
		  ':' => 'Colon',
		  '&ast;' => 'Ast',
		  '*' => 'Ast',
		  '&verbar;' => 'Verbar',
		  '|' => 'Verbar',
		  '&percnt;' => 'Percnt',
		  '%' => 'Percnt',
		 );
  if($lemma=~/^.*`([^0-9_-]+)/){
    $lemma=$1;
  }else{
    $lemma=~s/(.+?)[-_`].*$/$1/;
    if($lemma =~/^(?:(?:[ts]v|m)ůj|já|ty|jeho|se)$/){
      $lemma='#PersPron';
    }
    $lemma="#$specialEntity{$lemma}"if exists$specialEntity{$lemma};
  }
  $t_node->{'t_lemma'}=$lemma;
} #ANodeToALexRf

1;

=back

=cut
