BEGIN
{
	import TredMacro;
}

use strict;
use POSIX qw(ceil floor);

sub find_p_from_t_node
{
	my $t_node = shift;
	return unless $t_node;

	my $a_node = PML_T::GetANodeByID($t_node->attr('a/lex.rf'));
	return find_p_from_a_node($a_node);
}

sub find_p_from_a_node
{
	my $a_node = shift;
	return unless $a_node;
	my ($p_node) = grep { defined $_->{tag} } PML_En_A::GetPNodes($a_node);
	
	return $p_node;
}

sub find_a_node
{
	my ($p_id, $a_root) = @_;

	foreach my $a_node ($a_root->descendants)
	{
		return $a_node if $a_node->attr('p/terminal.rf') =~ m/p#$p_id$/;
	}

	return ();
}

sub find_t_from_a_node
{
	my ($a_node, $t_root) = @_;

	my $a_id = $a_node->{id};

	foreach my $t_node ($t_root->descendants)
	{
		return $t_node if $t_node->attr('a/lex.rf') =~ m/a#$a_id$/;
	}

	return undef;
}

sub find_t_node
{
	my ($p_id, $a_root, $t_root) = @_;

	my $a_node = find_a_node($p_id, $a_root);
	return () unless $a_node;

	return find_t_from_a_node($a_node, $t_root);
}

sub find_t_from_a_node_with_aux
{
	my ($a_node, $t_root) = @_;

	my $a_id = $a_node->{id};

	foreach my $t_node ($t_root->descendants)
	{
		return $t_node if $t_node->attr('a/lex.rf') =~ m/a#$a_id$/ or grep { /a#$a_id$/ } ListV($t_node->attr('a/aux.rf'));
	}

	return undef;
}

sub find_t_node_with_aux
{
	my ($p_id, $a_root, $t_root) = @_;

	my $a_node = find_a_node($p_id, $a_root);
	return () unless $a_node;

	return find_t_from_a_node_with_aux($a_node, $t_root);
}

sub hang_a_under_b
{
	my ($a, $b) = @_;

	#print STDERR "dep:\n";
	#print STDERR $a->parent->{id} . "\n";
	#print STDERR "gov:\n";
	#print STDERR $b->parent->{id} . "\n";

	return unless $a and $b;
	return if $a->parent == $b;

	if (grep { $b->{id} eq $_ } map { $_->{id} } $a->descendants)
	{
		CutPaste($b, $a->parent);
	}
	CutPaste($a, $b);
}

sub prevent_deepord_change
{
	my $r = shift;
	return unless $r and $r->{nodetype} eq 'root';

	foreach my $desc ($r->descendants)
	{
		$desc->{deepord} *= 1000;
	}
}

sub correct_deepord
{
	my $r = shift;
	return unless $r and $r->{nodetype} eq 'root';

	my $last_deepord = 0;
	$r->{deepord} = 0;
	my $node = $r;
	
	while ($node = NextNodeLinear($node, 'deepord'))
	{
		$node->{deepord} = ++$last_deepord;
	}
}

sub create_blank_t_node
{
	my $r = shift;
	return unless $r;

	my $n = PML_T::NewNode($r);
	$n->{nodetype} = 'qcomplex';
	$n->{functor} = '???';

	return $n;
}

sub create_t_from_a_node
{
	my ($p_id, $a_root, $t_root) = @_;
	return unless $p_id and $a_root;

	my $a_node = common::find_a_node($p_id, $a_root);
	my $a_parent = $a_node->parent;
	my $t_parent = ();
	while ($a_parent and not defined $t_parent)
	{
		$t_parent = common::find_t_from_a_node($a_parent, $t_root);
		$a_parent = $a_parent->parent;
	}
	($t_parent) = $t_root->children;
	@{$t_parent->{a}{'aux.rf'}} = ListSubtract($t_parent->{a}{'aux.rf'}, List('a#' . $a_node->{id})) if ListV($t_parent->attr('a/aux.rf'));

	my $new_t_node =  common::create_blank_t_node($t_parent);
	$new_t_node->{a}->{'lex.rf'} = 'a#' . $a_node->{id};
	$new_t_node->{deepord} = $t_parent->{deepord} - 1;

	return $new_t_node;
}

sub find_head
{
	my $node = shift;

	while ($node and defined $node->{phrase})
	{
		($node) = grep { $_->{is_head} > 0 } $node->children;
	}
	if ($node and defined $node->{tag}) { return $node }
	else { return }
}

sub remove_from_aux_node
{
	my ($t_node, $a_id) = @_;
	return unless $t_node and $a_id;

	@{$t_node->{a}{'aux.rf'}} = ListSubtract($t_node->{a}{'aux.rf'}, List('a#' . $a_id)) if ListV($t_node->attr('a/aux.rf'));
}

sub remove_from_aux_subtree
{
	my ($t_node, $a_id) = @_;
	return unless $t_node and $a_id;

	for my $node ($t_node->descendants)
	{
		remove_from_aux_node($node, $a_id);
	}
}

sub find_min_deepord
{
	my $t_node = shift;
	return unless $t_node;

	my $min = $t_node->{deepord};
	for my $child ($t_node->descendants)
	{
		$min = $child->{deepord} if $child->{deepord} < $min;
	}

	return $min;
}

sub find_max_deepord
{
	my $t_node = shift;
	return unless $t_node;

	my $max = $t_node->{deepord};
	for my $child ($t_node->descendants)
	{
		$max = $child->{deepord} if $child->{deepord} > $max;
	}

	return $max;
}

sub find_top_node
{
	my @nodes = @_;
	my $max = ();

	foreach my $node (@nodes)
	{
		next unless $node;
		if (not $max)
		{
			$max = $node;
			next;
		}

		$max = $node if not grep { $_->{id} eq $node->{id} } $max->descendants;
	}

	return $max;
}

sub find_p_top_node
{
	my @nodes = @_;
	my $max = ();

	foreach my $node (@nodes)
	{
		next unless $node;
		if (not $max) { $max = $node; next }

		while (not grep { $_->{id} eq $node->{id} } $max->descendants) { $max = $max->parent }
	}

	return $max;
}

#returns 1 if the node is terminal with tag=CC
#returns 2 if the node is nonterminal representing phrases "let alone" or "such as"
#returns 0 otherwise
sub is_CC
{
	my $p_node = shift;

	return 1 if $p_node->{tag} eq 'CC';
	if ($p_node->{phrase})
	{
		my @children = $p_node->{children};
		return 0 if scalar(@children) != 2;
		return 2 if $children[0]->{form} =~ m/^let$/i and $children[1]->{form} =~ m/^alone$/i;
		return 2 if $children[0]->{form} =~ m/^such$/i and $children[1]->{form} =~ m/^as$/i;
	}

	return 0;
}

1;

