# -*- cperl -*-

#ifndef PML_T_Anot2
#define PML_T_Anot2

#include <contrib/pml/PML_T_Edit.mak>

#encoding iso-8859-2

BEGIN { import PML_T; }

#include <contrib/support/unbind_edit.inc>

=pod

=head1 PML_T_Anot

PML_T_Anot.mak - Miscellaneous macros for annotating the
tectogrammatic layer in the way of Prague Dependency Treebank (PDT)
2.0.

=over 4

=cut

#key-binding-adopt PML_T_Edit
#menu-binding-adopt PML_T_Edit

#unbind-key e
#unbind-key s
#unbind-key G
#unbind-key N
#unbind-key P
#unbind-key F
#unbind-key t
#unbind-key Shift+space
#unbind-key Alt+j
#unbind-key j
#unbind-key J
#unbind-key Ctrl+j
#unbind-key Ctrl+space
#unbind-key space
#unbind-key A

#remove-menu Change is_parenthesis
#remove-menu Show valency frames and highlight assigned
#remove-menu Remember Node
#remove-menu Make Textual Coreference Arrow to Remembered Node
#remove-menu Jump to complement coreference antecedent
#remove-menu Jump to textual coreference antecedent
#remove-menu Annotate Special Coreference - Segment
#remove-menu Edit Node Type
#remove-menu Insert New Node
#remove-menu Edit TFA
#remove-menu Forget Remembered Node
#remove-menu Jump to grammatical coreference antecedent
#remove-menu Jump to coreference antecedent
#remove-menu Edit Grammatemes
#remove-menu Show effective ancestors
#remove-menu Annotate Special Coreference - Exophora


sub CreateStylesheets{
  SetStylesheetPatterns(<<'EOF','PML_T_Anot',1);
node:<? '#{customparenthesis}' if $${is_parenthesis}
  ?><? $${nodetype}eq'root' ? '${id}' : '${t_lemma}' ?><?
  $${annot_comment}?
  join('',' ',map{if(/valency/){'#{green}!'}elsif(/M-l/){'#{#506aff}M'}
    elsif(/T-lemma$/){'#{#0000e7}L'}elsif(/under/){'#{#0000e7}!'}
    elsif(/Typo/){'#{#506aff}!'}elsif(/Sent/){'#{brown}s'}
    elsif(/Word/){'#{brown}w'}elsif(/M-t/){'#{#506aff}t'}elsif(/Sec/){'#{red}#'}
    elsif(/Other|Functor|Structure/){'#{red}?'}elsif(/Not|Re/){'#{red}X'}
    elsif(/Vallex/){'#{green}?'}elsif(/Coref/){'#{orange}c'}
    else{'#{red}!'}
  }map{$_->{type}}ListV($this->{annot_comment}))
  :''?><? '#{customdetail}"' if $${is_dsp_root}
  ?><? '#{customdetail}.${sentmod}'if$${sentmod}
  ?><? '#{customcoref}'.$PML_T::coreflemmas_hash{$${id}}
    if $PML_T::coreflemmas_hash{$${id}}ne'' ?>

node:<?
  ($${nodetype} eq 'root' ? '#{customnodetype}${nodetype}' :
  '#{customfunc}${functor}').
  "#{customsubfunc}".($${subfunctor}?".\${subfunctor}":'').($${is_state}?".\${is_state=state}":'') ?><? '#{customcoappa}_${is_member=M}'if$${is_member} ?><? '#{customparenthesis}_${is_parenthesis=P}' if$${is_parenthesis} ?>

node:<? my $line;
        if(! $PML_T::showANodes){
          $line=(($this->attr('a/lex.rf')
            ? ('#{darkgreen}'
              .(PML_T::GetANodeByID($this->attr('a/lex.rf'))->attr('m/form')))
            : ''
          ).($this->attr('a/aux.rf')
            ? ('#{darkorange}'
              .join(' ','',map{
                  PML_T::GetANodeByID($_)->attr('m/form')
                }ListV($this->attr('a/aux.rf'))
              ))
            : ''
          ));
        }else{
          $line=(($${nodetype} ne 'complex' and $${nodetype} ne 'root')
            ? '#{customnodetype}${nodetype}'
            : '');
          $line.='#{customcomplex}${gram/sempos}';
        }
        return $line;
?>

style:#{Node-width:7}#{Node-height:7}#{Node-currentwidth:9}#{Node-currentheight:9}

style:<? '#{Node-shape:'.($this->{is_generated}?'rectangle':'oval').'}'?>

style:<? exists $PML_T::show{$${id}} ?'#{Node-addwidth:10}#{Node-addheight:10}':''?>

style:<?
  if(not ($${is_member}) and (
     ($this->{functor}=~/^(?:PAR|PARTL|VOCAT|RHEM|CM|FPHR|PREC)$/) or
     ($this->parent and $this->parent->{nodetype}eq'root'))) {
     '#{Line-width:1}#{Line-dash:2,4}#{Line-fill:'.CustomColor('line_normal').'}'
  } elsif ($${is_member}) {
    if (PML_T::IsCoord($this)and PML_T::IsCoord($this->parent)) {
      '#{Line-width:1}#{Line-fill:'.CustomColor('line_member').'}'
    } elsif ($this->parent and PML_T::IsCoord($this->parent)) {
      '#{Line-coords:n,n,(n+p)/2,(n+p)/2&(n+p)/2,(n+p)/2,p,p}#{Line-width:3&1}#{Line-fill:'.
       CustomColor('line_normal').'&'.CustomColor('line_member').'}'
    } else {
      '#{Line-fill:'.CustomColor('error').'}'
    }
  } elsif ($this->parent and PML_T::IsCoord($this->parent)) {
    '#{Line-width:1}#{Line-fill:'.CustomColor('line_comm').'}'
  } elsif (PML_T::IsCoord($this)) {
    '#{Line-coords:n,n,(n+p)/2,(n+p)/2&(n+p)/2,(n+p)/2,p,p}#{Line-width:1&3}#{Line-fill:'.
    CustomColor('line_member').'&'.CustomColor('line_normal').'}'
  } else {
    '#{Line-width:2}#{Line-fill:'.CustomColor('line_normal').'}'
  }
?>

style:<?
  if ($${gram/sempos}eq'v' and !$${val_frame.rf}
    and not first {$_->{type}eq'New valency frame'}ListV($this->{annot_comment})
  ){
    '#{Oval-fill:#00ffa0}'
  }elsif ($${tfa}=~/^[tfc]$/) {
    '#{Oval-fill:'.CustomColor('tfa_'.$${tfa}).'}${tfa}.'
  } else {
    '#{Oval-fill:'.CustomColor('tfa_no').'}'
  }
?>#{CurrentOval-width:3}#{CurrentOval-outline:<? CustomColor('current') ?>}

hint:<?
   my @hintlines;
   if (ref($this->{gram})) {
     foreach my $gram (sort keys %{$this->{gram}}) {
       push @hintlines, "gram/".$gram." : ".$this->{gram}->{$gram} if $this->{gram}->{$gram}
     }
   }
   push@hintlines, "is_dsp_root : 1" if $${is_dsp_root};
   push@hintlines, "is_name_of_person : 1" if $${is_name_of_person};
   push@hintlines, "quot : ". join(",",map{$_->{type}}ListV($this->{quot})) if $${quot};
   push @hintlines,map{'! '.$_->{type}.':'.$_->{text}}ListV($this->{annot_comment});
   join"\n", @hintlines
?>
EOF
}#CreateStylesheets

sub get_value_line_hook {
  my ($fsfile,$treeNo) = @_;
  return unless $fsfile;
  my $tree = $fsfile->tree($treeNo);
  return unless $tree;
  my @out = @{&PML_T::get_value_line_hook};
  if($tree->{eng_sentence}){
    push @out,(["\n".$tree->{eng_sentence},'-foreground=>lightblue']);
  }
  return \@out;
} # get_value_line_hook

sub switch_context_hook {
  CreateStylesheets();
  SetCurrentStylesheet('PML_T_Anot');
  Redraw() if GUI();
  undef$PML::arf;
  my ($prefix,$file)=FindVallex();
  ValLex::GUI::Init({-vallex_file => $file});
}

sub enable_edit_node_hook { 'stop' }

sub enable_attr_hook {
  if ($_[0]=~m!^(?:id|a|a/.*|compl\.rf.*|coref_gram\.rf.*|coref_text\.rf.*|deepord|quot|val_frame\.rf)$!)
    {'stop'} else {1}
}#enable_attr_hook

sub node_release_hook{
  my ($node,$target,$mod)=@_;
  if (&PML_T_Edit::node_release_hook eq 'stop'){
    return 'stop';
  }
  return if $mod || ! $target;
  if ($target->{t_lemma} eq "#Forn") {
    $node->{functor}='FPHR';
    $node->{nodetype}='fphr';
    $FileChanged=1;
  }
  return;
}#node_release_hook

sub get_status_line_hook {
  &PML_T_Edit::get_status_line_hook;
}#get_status_line_hook

sub status_line_doubleclick_hook {
  status_line_doubleclick( sub { ChooseValFrame($this) } ,@_);
} # status_line_doubleclick_hook

sub _Perm{
  my$pos=shift;
  my$perm=shift;
  if($pos<@_){
    for(my$i=$pos;$i<@_;$i++){
      if($i!=$pos){
        my$j=$_[$i];
        $_[$i]=$_[$pos];
        $_[$pos]=$j;
      }
      _Perm($pos+1,$perm,@_);
      if($i!=$pos){
        my$j=$_[$i];
        $_[$i]=$_[$pos];
        $_[$pos]=$j;
      }
    }
  }else{
    push @{$perm},join"_",@_;
  };
}#_Perm

sub _GenerateLemmaList{
  my$perm=[];
  _Perm(0,$perm,@_);
  return$perm;
}#_GenerateLemmaList

sub _TLemmaToEntity {
  my %Entities = ('%' => 'Percnt',
                  ',' => 'Comma',
                  ':' => 'Colon',
                  ';' => 'Semicolon',
                  '(' => 'Bracket',
                  '-' => 'Dash',
                  '.' => 'Period',
                  '/' => 'Slash',
                  '*' => 'Ast',
                  '&amp;' => 'Amp',
                  '&' => 'Amp',
                 );
  exists $Entities{$_[0]} ? '#'.$Entities{$_[0]} : $_[0];
} #_TLemmaToEntity

sub RegenerateTLemma{
  ChangingFile(0);
  my @anodes = GetANodes($this);
  if(@anodes>1){
    my@words=map{
      my $l = $_->attr('m/lemma');
      $l =~ s/(.+?)[-_`].*$/$1/;
      $l;
    }@anodes;
    if (@anodes == 3 and join('',@words) eq '...'){
      $this->{t_lemma} = '#Period3';
      ChangingFile(1);
      VallexWarning($this);
      return;
    }
    my$d=[$words[0]];
    ListQuery('Participating lemmas',
              'multiple',
              \@words,
              $d);
    my$possible=_GenerateLemmaList(@{$d});
    my$d=[$possible->[0]];
    ListQuery('New t-lemma','browse',
              $possible,
              $d) or return;
    $this->{t_lemma}=$d->[0];
    ChangingFile(1);
    VallexWarning($this);
  }elsif(@anodes==1){
    my $l = $anodes[0]->attr('m/lemma');
    $l =~ s/(.+?)[-_`].*$/$1/;
    $l = _TLemmaToEntity($l);
    $this->{t_lemma} = $l;
    ChangingFile(1);
    VallexWarning($this);
  }
}#RegenerateTLemma

sub TLemmaFromForms {
  ChangingFile(0);
  my @anodes = GetANodes($this);
  SortByOrd(\@anodes);
  my $newlemma = join '_', map { $_->attr('m/form') } @anodes;
  if ($newlemma ne $this->{t_lemma}){
    $this->{t_lemma}=$newlemma;
    ChangingFile(1);
    VallexWarning($this);
  }
}#TLemmaFromForms

sub AddComment {
  ChangingFile(0);
  my @list = $this->type('annot_comment/type/')->get_values();

  # for projects with no original sentences, Translation is not used
  @list = grep $_ ne 'Translation',@list
    unless defined $root->{eng_sentence} and length $root->{eng_sentence};

  # "Document split" comment should be kept for Companions and Malach
  @list = grep $_ ne 'Document split',@list
    unless $root->{id} =~ /^([a-z]{2}               # transcriber's name
                               (?:_[0-9]{3}         # czech companion files
                               |
                               -[0-9]{5}_[0-9]{2})) # malach files
                       \.[0-9]{2}-SCzechT-\1-/x;

  my $dialog = [$list[0]];
  ListQuery('Comment type',
            'browse',
            \@list,
            $dialog) or return;

  # For Re-generate, prepare the old sentence to be edited.
  my $default_text;
  if ($dialog->[0] eq 'Re-generate' and $this == $root){
    my $n = GetANodeByID($this->{'atree.rf'});
    $n = $n->root->following;
    my @sent=();
    while ($n) {
      push @sent,$n;
      $n = $n->following();
    }
    $default_text = join('',map{
      $_->attr('m/w/token').($_->attr('m/w/[-1]/no_space_after')?'':' ')
    } sort { $a->{ord} <=> $b->{ord} } @sent);
    $default_text =~ s/ *$//;
  }

  my $text = QueryString('Comment text','Text:',$default_text);
  return unless defined $text;
  my%comment=(type=>$dialog->[0],
              text=>$text);
  AddToList($this,'annot_comment',\%comment);
  ChangingFile(1);
}#AddComment

sub EditComment{
  ChangingFile(EditAttribute($this,'annot_comment'));
}#EditComment

sub AddNeg {
  $this=NewNode($this);
  $this->{functor}='RHEM';
  $this->{t_lemma}='#Neg';
  $this->{nodetype}='atom';
  $this->{is_generated}=1;
}#AddNeg

sub AddEmpVerb {
  $this=NewNode($this);
  $this->{functor}='PRED';
  $this->{t_lemma}='#EmpVerb';
  $this->{nodetype}='atom';
  $this->{is_generated}=1;
}#AddEmpVerb

sub AddEmpNoun {
  $this=NewNode($this);
  $this->{functor}='VOCAT';
  $this->{t_lemma}='#EmpNoun';
  $this->{nodetype}='atom';
  $this->{is_generated}=1;
}#AddEmpNoun

sub EditSubfunctor{
  ChangingFile(EditAttribute($this,'subfunctor'));
}#EditSubfunctor

sub EditFunctor{
  ChangingFile(EditAttribute($this,'functor'));
}#EditFunctor

sub EditSemPOS{
  ChangingFile(EditAttribute($this,'gram/sempos'));
}#EditSemPOS

sub RotateState{
    $this->{is_state}=!$this->{is_state};
}#RotateState

sub RotateDsp{
    $this->{is_dsp_root}=!$this->{is_dsp_root};
}#RotateDsp

## strip vallex PML-ref prefix from a given frame_rf
sub _stripped_frame_rf {
  my $frame_rf = shift;
  my $refid = FileMetaData('refnames')->{vallex};
  my @rf = map { my $x=$_;$x=~s/^\Q$refid\E#//; $x } AltV($frame_rf);
  return wantarray ? @rf : join '|',$frame_rf;
}

## get POS of the frame assigned to a given node
sub _assigned_frame_pos_of {
  my $node = shift || $this;
  return unless $node;
  if ($node->{'val_frame.rf'} ne q()) {
    my ($refid,$vallex_file) = FindVallex();
    my $V = ValLex::GUI::Init({-vallex_file=>$vallex_file});
    if ($V) {
      for my $id (_stripped_frame_rf($node->{'val_frame.rf'})) {
	my $frame = $V->by_id( $id );
	if ($frame) {
	  return lc($V->getPOS($V->getWordForFrame($frame)));
	}
      }
    }
  }
  return;
}

sub OpenValLexicon {
  shift unless @_ and ref($_[0]);
  my $node = shift || $this;
  my %opts = @_;
  $opts{-sempos}  ||= $node->attr('gram/sempos') || _assigned_frame_pos_of($node);
  PML_T::OpenValLexicon($node, %opts);
  ChangingFile(0);
}

sub ChooseValFrame {
  shift unless @_ and ref($_[0]);
  my $node = shift || $this;
  my %opts = @_;

  my $sempos = [ $opts{-sempos} || $node->attr('gram/sempos') || _assigned_frame_pos_of($node) ];
  if (!$sempos->[0]) {
    $sempos=['v'];
    ListQuery('Semantical POS','browse',[qw(v n)],$sempos) or return;
  }
  $opts{-sempos} = $sempos->[0];

  # valid frames for substituted ones
  my ($refid,$vallex_file) = FindVallex();
  my $V = ValLex::GUI::Init({-vallex_file => $vallex_file});
  my $valid_frame;
  if ($V) {
    for my $id (_stripped_frame_rf($node->{'val_frame.rf'})) {
      my $frame = $V->by_id( $id );
      if ($frame) {
        $frame = [ map $_->getAttribute('id'),$V->valid_frames_for($frame) ];
        $valid_frame = ($valid_frame ? "$valid_frame|" : '')
                       . join '|'
                       ,map _stripped_frame_rf($_),@$frame;
      }
    }
  }
  PML_T_Edit::ChooseValFrame($node, %opts, -sempos_attr => undef,-frameid => $valid_frame);
  ChangingFile(0);
} # ChooseValFrame

sub MarkForARf {
  $PML::arf=$this;
  ChangingFile(0);
  $PML::desiredcontext='PML_A_Edit';
  AnalyticalTree();
}#MarkForARf

sub VallexWarning {
  if($_[0]->{'val_frame.rf'}){
    questionQuery('T-lemma changed',
                  'T-lemma has changed. Verify that the vallex reference is correct.',
                  'OK');
  }
}#VallexWarning

sub after_edit_attr_hook {
  my($node,$attr,$result)=(shift,shift,shift);
  return unless $result;
  if($attr eq 't_lemma'){
    if($node->{t_lemma}=~m/^#(.*)/
       and not first{$1 eq $_} qw[Amp AsMuch Ast Benef Bracket Colon
         Comma Cor Dash EmpNoun EmpVerb Equal Forn Gen Idph Neg Oblfm
         Percnt Period Period3 PersPron QCor Rcp Semicolon Separ Slash
         Some Total Unsp]){
      questionQuery('Invalid entity',
                    'Cannot change t-lemma to undefined #-entity. T-lemma not changed.',
                    'OK');
      Undo();
    }else{
      VallexWarning($node);
    }
    if($node->{t_lemma}=~m/^#(?:Idph|Forn)$/){
      $node->{nodetype}='list';
    }elsif(($node->{t_lemma}=~m/^#(?:Amp|Ast|AsMuch|Cor|EmpVerb|Equal|Gen|Oblfm|Percnt|Qcor|Rcp|Some|Total|Unsp)$/o)
           or($node->{nodetype}ne'coap'
              and $node->{t_lemma}=~m/^#(?:Bracket|Comma|Colon|Dash|Period|Period3|Slash)$/o)){
      $node->{nodetype}='qcomplex';
    }
  }elsif($attr eq 'functor' and $node->parent){
    if(IsCoord($node)){
      $node->{nodetype}='coap';
    }elsif($node->{functor}=~/^(?:ATT|CM|INTF|MOD|PARTL|PREC|RHEM)$/){
      $node->{nodetype}='atom';
    }elsif($node->{functor}=~m/^([FD]PHR)$/){
      $node->{nodetype}=lc $1;
    }
  }
}#after_edit_attr_hook

sub value_line_click_hook {
  PML_T_Edit::MouseEditLinks(0,@_);
} # value_line_click_hook

sub EditTLemma{
  ChangingFile(EditAttribute($this,'t_lemma'));
  # Because of Undo in hook:
  $this=undef;
}#EditTLemma

sub AddANode {
  ChangingFile(0);
  PML_T_Edit::AddAnalyticNode(1);
}#AddANode

sub AddENode {
  ChangingFile(0);
  PML_T_Edit::AddEntityNode(1);
}#AddENode

=item DeleteNodeToAux(node?)

Deletes $node or $this, attaches all its children and references from
a/ to its parent and recounts deepord. Cannot be used for the root.

=cut

sub DeleteNodeToAux{
  shift unless ref $_[0];
  my$node=$_[0]||$this;
  ChangingFile(0),return unless $node->parent;
  my$parent=$node->parent;
  foreach my$child($node->children){
    CutPaste($child,$parent);
  }
  AddToList($parent,
            'a/aux.rf',
            grep { defined $_ }
            $node->attr('a/lex.rf'),ListV($node->attr('a/aux.rf'))
           ) if $node->attr('a/lex.rf') or $node->attr('a/aux.rf');
  DeleteLeafNode($node);
  $this=$parent unless@_;
  ChangingFile(1);
}#DeleteNodeToAux

sub RotateParenthesisSubtree{
  my $par = ! $this->{is_parenthesis};
  foreach my $node ($this,$this->descendants) {
    $node->{is_parenthesis} = $par;
  }
}#RotateParenthesisSubtree

sub AddForn {
  $this=NewNode($this);
  $this->{functor}='ID';
  $this->{t_lemma}='#Forn';
  $this->{nodetype}='list';
  $this->{is_generated}=1;
}#AddForn

sub AddNewNode {
  $this=NewNode($this);
  $this->{functor}='VOCAT';
  $this->{t_lemma}='#NewNode';
  $this->{nodetype}='qcomplex';
  $this->{is_generated}=1;
}#AddNewNode

sub ClearValframe {
  undef $this->{'val_frame.rf'};
}#ClearValframe


sub AssignFunctor {
  $this->{functor}=$_[0];
  after_edit_attr_hook($this,'functor',$_[0]);
}#AssignFunctor

sub FuncRSTR {
  AssignFunctor('RSTR');
}#FuncRSTR

sub FuncACT {
  AssignFunctor('ACT');
}#FuncACT

sub FuncPAT {
  AssignFunctor('PAT');
}#FuncPAT

sub FuncPRED {
  AssignFunctor('PRED');
}#FuncPRED

sub FuncCONJ {
  AssignFunctor('CONJ');
}#FuncCONJ

sub FuncLOC {
  AssignFunctor('LOC');
}#FuncLOC

sub FuncTWHEN {
  AssignFunctor('TWHEN');
}#FuncTWHEN

sub FuncRHEM {
  AssignFunctor('RHEM');
}#FuncRHEM

sub FuncDENOM {
  AssignFunctor('DENOM');
}#FuncDENOM

sub FuncMANN {
  AssignFunctor('MANN');
}#FuncMANN

sub FuncEXT {
  AssignFunctor('EXT');
}#FuncEXT

sub FuncID {
  AssignFunctor('ID');
}#FuncID

sub FuncMAT {
  AssignFunctor('MAT');
}#FuncMAT

sub FuncDIR3 {
  AssignFunctor('DIR3');
}#FuncDIR3

sub FuncDIR1 {
  AssignFunctor('DIR1');
}#FuncDIR1

sub FuncBEN {
  AssignFunctor('BEN');
}#FuncBEN

sub FuncFPHR {
  AssignFunctor('FPHR');
}#FuncFPHR

sub FuncPREC {
  AssignFunctor('PREC');
}#FuncPREC

sub FuncCRIT {
  AssignFunctor('CRIT');
}#FuncCRIT

sub FuncCAUS {
  AssignFunctor('CAUS');
}#FuncCAUS

sub FuncREG {
  AssignFunctor('REG');
}#FuncREG

sub FuncTHL {
  AssignFunctor('THL');
}#FuncTHL

sub FuncADVS {
  AssignFunctor('ADVS');
}#FuncADVS

sub FuncTHO {
  AssignFunctor('THO');
}#FuncTHO

sub FuncCOMPL {
  AssignFunctor('COMPL');
}#FuncCOMPL

sub FuncCOND {
  AssignFunctor('COND');
}#FuncCOND

sub FuncOPER {
  AssignFunctor('OPER');
}#FuncOPER

sub FuncAUTH {
  AssignFunctor('AUTH');
}#FuncAUTH

sub FuncCSQ {
  AssignFunctor('CSQ');
}#FuncCSQ

sub FuncSUBS {
  AssignFunctor('SUBS');
}#FuncSUBS

sub FuncTFRWH {
  AssignFunctor('TFRWH');
}#FuncTFRWH

sub FuncTOWH {
  AssignFunctor('TOWH');
}#FuncTOWH

sub FuncDIR2 {
  AssignFunctor('DIR2');
}#FuncDIR2

sub FuncVOCAT {
  AssignFunctor('VOCAT');
}#FuncVOCAT

sub FuncAPP {
  AssignFunctor('APP');
}#FuncAPP

sub FuncADDR {
  AssignFunctor('ADDR');
}#FuncADDR

sub FuncEFF {
  AssignFunctor('EFF');
}#FuncEFF

sub FuncPAR {
  AssignFunctor('PAR');
}#FuncPAR

sub FuncAPPS {
  AssignFunctor('APPS');
}#FuncAPPS

sub FuncACMP {
  AssignFunctor('ACMP');
}#FuncACMP

sub FuncAIM {
  AssignFunctor('AIM');
}#FuncAIM

sub FuncMEANS {
  AssignFunctor('MEANS');
}#FuncMEANS

sub FuncCM {
  AssignFunctor('CM');
}#FuncCM

sub FuncDIFF {
  AssignFunctor('DIFF');
}#FuncDIFF

sub FuncCPHR {
  AssignFunctor('CPHR');
}#FuncCPHR

sub FuncCPR {
  AssignFunctor('CPR');
}#FuncCPR

sub FuncTSIN {
  AssignFunctor('TSIN');
}#FuncTSIN

sub FuncTPAR {
  AssignFunctor('TPAR');
}#FuncTPAR

sub FuncORIG {
  AssignFunctor('ORIG');
}#FuncORIG

sub FuncMOD {
  AssignFunctor('MOD');
}#FuncMOD

sub FuncATT {
  AssignFunctor('ATT');
}#FuncATT

sub FuncCNCS {
  AssignFunctor('CNCS');
}#FuncCNCS

sub FuncCONFR {
  AssignFunctor('CONFR');
}#FuncCONFR

sub FuncCONTRA {
  AssignFunctor('CONTRA');
}#FuncCONTRA

sub FuncCONTRD {
  AssignFunctor('CONTRD');
}#FuncCONTRD

sub FuncDISJ {
  AssignFunctor('DISJ');
}#FuncDISJ

sub FuncDPHR {
  AssignFunctor('DPHR');
}#FuncDPHR

sub FuncGRAD {
  AssignFunctor('GRAD');
}#FuncGRAD

sub FuncHER {
  AssignFunctor('HER');
}#FuncHER

sub FuncINTF {
  AssignFunctor('INTF');
}#FuncINTF

sub FuncINTT {
  AssignFunctor('INTT');
}#FuncINTT

sub FuncPARTL {
  AssignFunctor('PARTL');
}#FuncPARTL

sub FuncREAS {
  AssignFunctor('REAS');
}#FuncREAS

sub FuncRESL {
  AssignFunctor('RESL');
}#FuncRESL

sub FuncRESTR {
  AssignFunctor('RESTR');
}#FuncRESTR

sub FuncTFHL {
  AssignFunctor('TFHL');
}#FuncTFHL

sub FuncTTILL {
  AssignFunctor('TTILL');
}#FuncTTILL


#bind RegenerateTLemma to Alt+L menu Regenerate T-lemma
#bind TLemmaFromForms to Alt+F menu Generate T-lemma from Forms
#bind AddComment to ! menu Add Annotator's comment
#bind EditComment to ? menu Edit Annotator's Comment
#bind AddNeg to Ctrl+h menu Add Negation
#bind AddEmpVerb to Ctrl+r menu Add EmpVerb
#bind AddEmpNoun to Ctrl+u menu Add EmpNoun
#bind EditFunctor to Ctrl+f menu Edit Functor
#bind RotateState to Alt+S menu Rotate State
#bind RotateDsp to Alt+d menu Rotate is_dsp_root
#bind OpenValLexicon to Ctrl+Shift+Return menu Browse valency frame lexicon
#bind ChooseValFrame to Ctrl+Return menu Select and assign valency frame
#bind MarkForARf to + menu Mark for reference changes and enter A-layer
#bind EditTLemma to Alt+l menu Edit t_lemma
#bind AddANode to Shift+Insert menu Insert New Node from A-layer
#bind AddENode to Insert menu Insert New #-Entity Node
#bind DeleteNodeToAux to Shift+Delete menu Delete Node Moving to Aux
#bind PML_T_Edit->RotateParenthesis to Alt+P menu Change is_parenthesis
#bind RotateParenthesisSubtree to Alt+p menu Change is_parenthesis for Subtree
#bind AddForn to Ctrl+i menu Add #Forn.ID
#bind AddNewNode to Ctrl+w menu Add #NewNode.VOCAT
#bind ToggleANodes to Ctrl+a menu Toggle display of a-nodes or grammatemes
#bind PML_T_Edit->RotateGenerated to Alt+r menu Change is_generated
#bind PML_T_Edit->RotateMember to Alt+m menu Change is_member
#bind ClearValframe to Ctrl+F menu Clear Valency Frame

#bind FuncRSTR to r menu Assign RSTR
#bind FuncACT to A menu Assign ACT
#bind FuncPAT to P menu Assign PAT
#bind FuncPRED to p menu Assign PRED
#bind FuncCONJ to c menu Assign CONJ
#bind FuncLOC to l menu Assign LOC
#bind FuncTWHEN to t menu Assign TWHEN
#bind FuncRHEM to h menu Assign RHEM
#bind FuncDENOM to n menu Assign DENOM
#bind FuncMANN to m menu Assign MANN
#bind FuncEXT to x menu Assign EXT
#bind FuncID to I menu Assign ID
#bind FuncMAT to M menu Assign MAT
#bind FuncDIR3 to 3 menu Assign DIR3
#bind FuncDIR1 to 1 menu Assign DIR1
#bind FuncBEN to b menu Assign BEN
#bind FuncFPHR to 6 menu Assign FPHR
#bind FuncPREC to e menu Assign PREC
#bind FuncCRIT to K menu Assign CRIT
#bind FuncCAUS to C menu Assign CAUS
#bind FuncREG to g menu Assign REG
#bind FuncTHL to u menu Assign THL
#bind FuncADVS to 7 menu Assign ADVS
#bind FuncTHO to v menu Assign THO
#bind FuncCOMPL to k menu Assign COMPL
#bind FuncCOND to o menu Assign COND
#bind FuncOPER to 0 menu Assign OPER
#bind FuncAUTH to U menu Assign AUTH
#bind FuncCSQ to q menu Assign CSQ
#bind FuncSUBS to B menu Assign SUBS
#bind FuncTFRWH to W menu Assign TFRWH
#bind FuncTOWH to w menu Assign TOWH
#bind FuncDIR2 to 2 menu Assign DIR2
#bind FuncVOCAT to F9 menu Assign VOCAT
#bind FuncAPP to a menu Assign APP
#bind FuncADDR to D menu Assign ADDR
#bind FuncEFF to E menu Assign EFF
#bind FuncPAR to V menu Assign PAR
#bind FuncAPPS to 8 menu Assign APPS
#bind FuncACMP to Y menu Assign ACMP
#bind FuncAIM to i menu Assign AIM
#bind FuncMEANS to s menu Assign MEANS
#bind FuncCM to X menu Assign CM
#bind FuncDIFF to d menu Assign DIFF
#bind FuncCPHR to 4 menu Assign CPHR
#bind FuncCPR to j menu Assign CPR
#bind FuncTSIN to z menu Assign TSIN
#bind FuncTPAR to y menu Assign TPAR
#bind FuncORIG to O menu Assign ORIG
#bind FuncMOD to Q menu Assign MOD
#bind FuncATT to T menu Assign ATT
#bind FuncCNCS to N menu Assign CNCS
#bind FuncCONFR to 9 menu Assign CONFR
#bind FuncCONTRA to Ctrl+x menu Assign CONTRA
#bind FuncCONTRD to Ctrl+y menu Assign CONTRD
#bind FuncDISJ to J menu Assign DISJ
#bind FuncDPHR to 5 menu Assign DPHR
#bind FuncGRAD to G menu Assign GRAD
#bind FuncHER to H menu Assign HER
#bind FuncINTF to F menu Assign INTF
#bind FuncINTT to Z menu Assign INTT
#bind FuncPARTL to F1 menu Assign PARTL
#bind FuncREAS to F8 menu Assign REAS
#bind FuncRESL to R menu Assign RESL
#bind FuncRESTR to S menu Assign RESTR
#bind FuncTFHL to f menu Assign TFHL
#bind FuncTTILL to L menu Assign TTILL

1;

=back

=cut

#endif
