# -*- cperl -*-

package English_to_Czech;
#binding-context English_to_Czech
#key-binding-adopt TredMacro

BEGIN { import TMT; }

push @TredMacro::AUTO_CONTEXT_GUESSING, sub {
	my ($hook)=@_;
	my $resuming = ($hook eq 'file_resumed_hook');
	my $current = CurrentContext();
	if (TMT::is_tmt_file()) {
    	SetCurrentStylesheet('English_to_Czech') if $resuming;
    	return 'English_to_Czech';
	}
	return;
};

sub allow_switch_context_hook {
  return 'stop' if not TMT::is_tmt_file();
}

sub switch_context_hook {
  SetCurrentStylesheet('English_to_Czech');
  Redraw() if GUI();
}

sub get_value_line_hook {
  my ($fsfile,$treeNo)=@_;
  return unless $fsfile;
  my $bundle = $fsfile->tree($treeNo);
  return unless $bundle;

  my $en_s = $bundle->attr('english_source_sentence');
  my $cs_s = $bundle->attr('czech_source_sentence');
  my $cs_t = $bundle->attr('czech_target_sentence'); 
  
  if ($en_s || $cs_s || $cs_t) {
  	return $en_s."\n". $cs_s ."\n". $cs_t;
  } else {
      my @sentences = ();
	  my @t_names = qw(SEnglishM SCzechM);
	  
	  foreach my $t_name (@t_names) {
	  	my $tree = $bundle->attr("trees/$t_name");
	  	if (defined $tree) {
	  		my $s = join "", map { $_->{form} . ($_->{no_space_after} ? '': ' ') } $tree->children;
	  		$s =~ s/\s+$//;
	  		push (@sentences, $s)
	  	}
	  }
	  
	  return join("\n", @sentences);
  }
}

sub get_nodelist_hook {
  my ($fsfile, $treeNo, $currentNode, $hidden)=@_;
  my $bundle = $fsfile->tree($treeNo);
  my $ord = FS()->order;

  if ($bundle->attr('trees/SEnglishP')) {
    TMT::compute_ord($bundle->attr('trees/SEnglishP'));
  }

  my @nodes = get_all_nodes_to_display($bundle);

  #print "Ord uzlu:\n";
  #print join " ",map{"o=$_->{ord}"} @nodes;
  #print "\n";
  $currentNode=$nodes[0] unless first { $_==$currentNode } @nodes;
  return [\@nodes, $currentNode];
}

sub get_all_nodes_to_display {
  my $bundle = shift;
  return TMT::get_all_nodes_to_display($bundle);
}

sub after_redraw_hook {
  return if !defined $root; # no warnings when using Tred's "reload current file" button
  my $tree = $root->attr('trees/TCzechA');

  if (defined $tree) {
    my %groups;

    for my $node ($tree,$tree->descendants) {
      my $group = $node->{clause_number};
      if (defined $group) {
        push @{$groups{ $group }}, $node;
      }
    }

    TrEd::NodeGroups::draw_groups(
				  $grp,
				  [ map { $groups{$_} } sort {$a<=>$b} keys %groups ],
				  {
				   colors  => $colors,
				   stipples => TrEd::NodeGroups::dense_stipples($grp),
				   x_shift => 0,
				  }
				 );
    my $stipple;

    my $sub = sub {
      my ($w,$in)=@_;
      my ($group_tag) = grep { /group_no_/ } $w->gettags('current');
      if (defined $group_tag) {
	if ($in) {
	  $stipple = $w->itemcget($group_tag,'-stipple');
	  $w->itemconfigure($group_tag,-stipple => undef);
	} else {
	  $w->itemconfigure($group_tag,-stipple => $stipple);
	  undef $stipple;
	}
      }
    };

    my $c = $grp->treeView->realcanvas;
    $c->bind('group_line','<Enter>',[$sub,1]);
    $c->bind('group_line','<Leave>',[$sub,0]);
  }

  $tree = $root->attr('trees/SEnglishT');

  if (defined $tree) {
    my %groups;
    for my $node ($tree,$tree->descendants) {
      my $group = $node->{named_entity}->{number};
      if (defined $group) {
		  #my $c = $ne_type_colors{$node->{named_entity}->{type}};
		  #print STDERR "$c used for " . $node->{t_lemma} . "\n";
	      push @{$groups{ $group }}, $node;
      }
    }

    TrEd::NodeGroups::draw_groups(
				  $grp,
				  [ map { $groups{$_} } sort {$a<=>$b} keys %groups ],
				  {
				   colors  => [ map {$ne_type_colors{$groups{$_}->[0]->{named_entity}->{type}}} sort {$a<=>$b} keys %groups ],
				   stipples => TrEd::NodeGroups::dense_stipples($grp),
				   x_shift => 0,
				  }
				 );
    my $stipple;

    my $sub = sub {
      my ($w,$in)=@_;
      my ($group_tag) = grep { /group_no_/ } $w->gettags('current');
      if (defined $group_tag) {
	if ($in) {
	  $stipple = $w->itemcget($group_tag,'-stipple');
	  $w->itemconfigure($group_tag,-stipple => undef);
	} else {
	  $w->itemconfigure($group_tag,-stipple => $stipple);
	  undef $stipple;
	}
      }
    };

    my $c = $grp->treeView->realcanvas;
    $c->bind('group_line','<Enter>',[$sub,1]);
    $c->bind('group_line','<Leave>',[$sub,0]);
  }
}

sub wmt_display_line ($$) {  # pouze pro ucely WMT
  my ($node, $line) = @_;

  if (not $node->parent) {
    if ($line==1) {
#      return "$node->{_tree_name}";
      my $tree_name =  $node->{_tree_name};
      if ($tree_name =~ /^(.)(.+)(.)$/) {
	return "$2 ".lc($3)."-layer";
      }
      else {
	return "";
      }

    }
  }

  else {
    if ($node->{_tree_name} eq "SEnglishP") {
      if ($line==1) {
#	return "$node->{tag} $node->{phrase}";
	return ($node->{phrase} || $node->{form});
      }
      elsif ($line==2) {
	return undef;
	if ($node->{form}) {
	  return "$node->{form}";
	}
	elsif ($node->{is_head}) {
	  return "#{darkgrey}($node->{head_selection_rule})";
	}
      }
      elsif ($line==3) {
	return undef;
	if ($node->{form} and $node->{is_head}) {
	  return "#{darkgrey}($node->{head_selection_rule})";
	}
      }

    }

    elsif ($node->{_tree_name}=~/SCzechA$/) {
      if ($line==1) {
	return $node->attr('m/form');
      } elsif ($line==2) {
	return $node->attr('afun').($node->{is_member}?"_Co":"");
      }
    }


    elsif ($node->{_tree_name}=~/TCzechA$/) {
      if ($line==2) {
	return (join "",map {$node->attr("morphcat/$_")} @morphcats).($node->{is_member}?" Co":"");
      }
      elsif ($line==1) {
	return $node->attr('m/form');
      }
      elsif ($line==3) {
	return $node->attr('afun');
      }
    }



    elsif ($node->{_tree_name}=~/A$/) {
      if ($line==3) {
	return $node->attr('m/tag').($node->{is_member}?"_Co":"");
      }
      elsif ($line==1) {
	return $node->attr('m/form');
      }
      elsif ($line==2) {
	return $node->attr('afun');
      }
    }

    elsif ($node->{_tree_name}=~/M$/) {
      if ($line==1) {
	return "$node->{form}";
      }
      elsif ($line==2) {
	return "#{darkgreen}$node->{lemma} #{darkgreen}$node->{tag}";
      }
      elsif ($line==3) {
	return "#{darkgreen}$node->{tag}";
      }
    }

    elsif ($node->{_tree_name}=~/T(English|Czech)T/) {
      if ($line==1) {
	return "$node->{t_lemma}";
      }
      elsif ($line==2) {
	return "$node->{formeme}";
	my $color = ($node->{t_lemma} eq $node->{loglin_tlemma}) ? "#{blue}" : "#{black}";
	return "${color}$node->{loglin_tlemma} / $node->{loglin_formeme}";
      }
      elsif ($line==3) {
	return "$node->{functor}".($node->{is_member}?"_CO":"");
      }
    }


    elsif ($node->{_tree_name}=~/(English|Czech)T/) {
      if ($line==1) {
	return "$node->{t_lemma}";
      }
      elsif ($line==3) {
	return "$node->{functor}".($node->{is_member}?"_CO":"");
      }
      elsif ($line==2) {
	return "#{black}$node->{formeme}"
      }
    }

    else {
      return "???";
    }
  }
}

# ------------ pridano kvuli vykreslovani koreferenci -----------------

sub SearchForNodeById {
  my $id = shift;
  my @nodes = get_all_nodes_to_display($root);
  my $selected_node = grep {$_->{id} eq $id} @nodes;
  if ($selected_node) {
    return ($selected_node,16);
  }
}


# vykopirovano z PML_T.mak:

sub node_style_hook {
  my ($node,$styles)=@_;
  my %line = GetStyles($styles,'Line');
  my %my_line;

#  if ($node->{'coref_gram.rf'}) {
#    $node->{coref_special} = 'exoph';
#  }


  # make sure we only alter the very first line
  my $lines = scalar(split /&/,$line{coords});
  if ($lines>1) {
    for (keys %my_line) {
      my @l = split /&/,$line{$_};
      shift @l;
      $line{$_}=join '&',$my_line{$_},@l;
    }
  } else {
    $line{$_}=$my_line{$_} for keys %my_line;
  }

  if  ($node->{'coref_text.rf'} or $node->{'coref_gram.rf'} or $node->{'compl.rf'} or $node->{coref_special}) {
    my @gram = grep {$_ ne "" } ListV($node->{'coref_gram.rf'});
    my @text = grep {$_ ne "" } ListV($node->{'coref_text.rf'});
    my @compl = grep {$_ ne "" } ListV($node->{'compl.rf'});
    DrawCorefArrows($node,$styles,\%line,
		      [@gram,@text,@compl],
		      [(map 'grammatical',@gram),
		       (map 'textual',@text),
		       (map 'compl',@compl)
		      ]
                     );
  }


  # Draw alignment on the tectogramatical layer
  # There are three types of alignment arrows:
  # blue: aligns made both manually and automatically
  # red:  aligns made only manually
  # green: aligns made only automatically
  if ($node->attr('align/links') or $node->attr('align/manual_links')) {
    my %automatic;
    my $links_rf = $node->attr('align/links');
    for(my $i = 0; $i <= $#$links_rf; $i++) {
      $automatic{$links_rf->[$i]->{'counterpart.rf'}} = 1;
    }
    my %manual;
    my $manual_links_rf = $node->attr('align/manual_links');
    for(my $i = 0; $i <= $#$manual_links_rf; $i++) {
      $manual{$manual_links_rf->[$i]->{'counterpart.rf'}} = 1;
    }
    my @both;
    my @automatic_only;
    my @manual_only;
    foreach my $a (keys %automatic) {
      if ($manual{$a}) {
        push(@both, $a);
      }
      else {
        push(@automatic_only, $a);
      }
    }
    foreach my $m (keys %manual) {
      if (not $automatic{$m}) {
        push(@manual_only, $m);
      }
    }
    DrawCorefArrows($node,$styles,\%line,
		      [@both,@automatic_only,@manual_only],
		      [(map 'textual', @both),
		       (map 'compl', @automatic_only),
		       (map 'grammatical', @manual_only)
		      ]
                     );
  }

  1;
}

# = DrawCorefArrows()

# Called from C<node_style_hook>. Draws coreference arrows using
# following properties: textual arrows in CustomColor C<arrow_textual>,
# grammatical in <arrow_grammatical> (and dashed in Full stylesheet),
# complement arrow in C<arrow_compl> (and dot-dashed in Full
# stylesheet), segment arrow in C<arrow_segm> and exophora arrow in
# C<arrow_exoph>.

# =cut

#$main::macroDebug=1;

sub DrawCorefArrows {
  my ($node,$styles,$line,$corefs,$cortypes)=@_;
  delete $coreflemmas{$node->{id}};
  my (@coords,@colors,@dash,@tags);
  my ($rotate_prv_snt,$rotate_nxt_snt,$rotate_dfr_doc)=(0,0,0);
  my $ids={};
#  my $nd = $root; while ($nd) { $ids->{$nd->{id}}=1 } continue { $nd=$nd->following };  # nahrazeno by ZZ nasledujicim radkem
#  foreach my $node (get_all_nodes_to_display($root)) { $ids->{$node->{id}}=1 };         # by PP: rychlejsi a funkcni pri tisku pomoci example/print.btred
  foreach my $node (GetDisplayedNodes()) { $ids->{$node->{id}}=1 };
  foreach my $coref (@$corefs) {
    my $cortype=shift @$cortypes;

#    print "Coref: $coref    cortype: $cortype\n";
    if ($ids->{$coref}) { #index($coref,$id1)==0) {
      print STDERR "ref-arrows: Same sentence\n" if $main::macroDebug;
      # same sentence
      my $T="[?\$node->{id} eq '$coref'?]";
      my $X="(x$T-xn)";
      my $Y="(y$T-yn)";
      my $D="sqrt($X**2+$Y**2)";
      push @colors,CustomColor('arrow_'.$cortype);
      push @dash,GetPatternsByPrefix('full')?($cortype eq'compl'?'2,3,5,3':$cortype eq'grammatical'?'5,3':qw()):qw();


      my $c = <<COORDS;

&n,n,
(x$T+xn)/2 - $Y*(25/$D+0.12),
(y$T+yn)/2 + $X*(25/$D+0.12),
x$T,y$T


COORDS
      push @coords,$c;
      push @tags,$cortype;
    } else { # should be always the same document, if it exists at all
      my($refed,$treeNo)=SearchForNodeById($coref);
      my $orientation=$treeNo-CurrentTreeNumber()-1;
      $orientation=$orientation>0 ? 'right' : ($orientation<0 ? 'left':0);
      $coreflemmas{$node->{id}}.=' '.$refed->{t_lemma};
      if($orientation=~/left|right/){
	if($orientation eq'left'){
	  print STDERR "ref-arrows: Preceding sentence\n"if $main::macroDebug;
	  push @colors,CustomColor('arrow_'.$cortype);
	  push @dash,qw();
	  push @coords,"\&n,n,n-30,n+$rotate_prv_snt";
	  push @tags,$cortype;
	  $rotate_prv_snt+=10;
	}else{ #right
	  print STDERR "ref-arrows: Following sentence\n" if $main::macroDebug;
	  push @colors,CustomColor('arrow_'.$cortype);
	  push @dash,qw();
	  push @coords,"\&n,n,n+30,n+$rotate_nxt_snt";
	  push @tags,$cortype;
	  $rotate_nxt_snt+=10;
	}
      }else{
	print STDERR "ref-arrows: Not found!\n" if $main::macroDebug;
	push @colors,CustomColor('error');
	push @dash,qw();
	push @coords,"&n,n,n+$rotate_dfr_doc,n-25";
	push @tags,$cortype;
	$rotate_dfr_doc+=10;
      }
    }
  }
  if($node->{coref_special}eq'segm') { # pointer to an unspecified segment of preceeding sentences
    print STDERR "ref-arrows: Segment - unaimed arrow\n" if $main::macroDebug;
    push @colors,CustomColor('arrow_segment');
    push @dash,qw();
    push @coords,"&n,n,n-25,n+$rotate_prv_snt";
    push @tags,'segm';
    $rotate_prv_snt+=10;
  }
  if($node->{coref_special}eq'exoph') {
    print STDERR "ref-arrows: Exophora\n" if $main::macroDebug;
    push @colors,CustomColor('arrow_exoph');
    push @dash,qw();
    push @coords,"&n,n,n+$rotate_dfr_doc,n-25";
    push @tags,'exoph';
    $rotate_dfr_doc+=10;
  }
  $line->{-coords} ||= 'n,n,p,p';

  # make sure we don't alter any previous line
  my $lines = scalar($line->{-coords}=~/&/g)+1;
  for (qw(-arrow -dash -width -fill -smooth -arrowshape)) {
    $line->{$_}.='&'x($lines-scalar($line->{$_}=~/&/g)-1);
  }
  if (@coords) {
    AddStyle($styles,'Line',
	     -coords => $line->{-coords}.join("",@coords),
	     -arrow => $line->{-arrow}.('&last' x @coords),
             -arrowshape => $line->{-arrowshape}.('&16,18,3' x @coords),
	     -dash => $line->{-dash}.join('&','',@dash),
	     -width => $line->{-width}.('&1' x @coords),
	     -fill => $line->{-fill}.join("&","",@colors),
	     -tag => $line->{-tag}.join("&","",@tags),
	     -smooth => $line->{-smooth}.('&1' x @coords)
	    );
  }
}

#bind jump_to_rightmost_tree_in_next_bundle to n
sub jump_to_rightmost_tree_in_next_bundle {
  print STDERR "Jumping to the previous bundle: ". ($grp->{treeNo}+2)."\n";
  GotoTree($grp->{treeNo}+2);
  my @roots = grep {not $_->parent} get_all_nodes_to_display($root);
  $this = $roots[-1]

}

#bind jump_to_rightmost_tree_in_prev_bundle to p
sub jump_to_rightmost_tree_in_prev_bundle {
  print STDERR "Jumping to the next bundle: ".($grp->{treeNo})."\n";
  GotoTree($grp->{treeNo});
  my @roots = grep {not $_->parent} get_all_nodes_to_display($root);
  $this = $roots[-1]

}

