# -*- cperl -*-

#include <contrib/pml/PML.mak>

package TMT;

BEGIN { import TredMacro; }

our ($this, $root, $grp);
use List::Util qw(first);


# detect that a file is a tmt document
sub is_tmt_file {
  return (((PML::SchemaName()||'') eq 'tmt_document') ? 1 : 0);
}

sub compute_ord($) {
  my $node = shift;
  my @children = $node->children;
  if (@children) {
    my $sum;
    foreach my $child (@children) {
      $sum += compute_ord($child);
    }
    $node->{_ord} = $sum / @children;
    } else {
    $node->{id} =~ /(\d+)$/;
    $node->{_ord} = $1;
  }
  return $node->{_ord};

}

my $colors = [qw(#CF9 #FDA #9FF #FAF #AFA #AAF #FAA)];
my %ne_type_colors = ( 'location' => "#FAA", 'person' => '#AAF', 'organization' => '#AFA');

sub switch_stylesheet_hook {
  # we may have loaded a file, and then switched
  # to one of our tmt stylesheets
  _prepare_tree_names();
}



sub _interleave {
    my @lists = @_;
    my $node_exists = 1;
    my @result = ();
    while ($node_exists) {
        $node_exists = 0;
        foreach my $list (@lists) {
            if (@$list) {
                push @result, shift(@$list);
                $node_exists = 1;
            }
        }
    }
  return @result;
}

my @languages;

sub group_nodes_by_direction_and_layer {
    my $bundle = shift;

    # get sorted list of lanuages
    my %hash_tree_names;
    foreach my $tree (keys %{$bundle->{trees}}) {
        $tree =~ s/^[ST](.+)[MPAT]$/$1/;
        $hash_tree_names{$tree} = 1;
    }

    @languages = keys %hash_tree_names;

    # named entities trees
    my @sn_trees;
    foreach my $lang (@languages) {
        push @sn_trees, [map {($_, $_->descendants)}  grep {$_} $bundle->attr('trees/S${lang}N')];
    }

    # source morphological trees
    my @sm_trees;
    foreach my $lang (@languages) {
        push @sm_trees, [map {($_,$_->descendants)} grep {$_} $bundle->attr("trees/S${lang}M")];
    }

    # source phrase trees
    my @sp_trees;
    foreach my $lang (@languages) {
        push @sp_trees, [sort {$a->{_ord} <=> $b->{_ord}} map {($_, $_->descendants)}  grep {$_} $bundle->attr('trees/S${lang}P')];
    }

    # source analytical trees
    my @sa_trees;
    foreach my $lang (@languages) {
        push @sa_trees, [sort {$a->{ord} <=> $b->{ord}} map {($_,$_->descendants)} grep {$_} $bundle->attr("trees/S${lang}A")];
    }

    # source tectogrammatical trees
    my @st_trees;
    foreach my $lang (@languages) {
        push @st_trees, [sort {$a->{deepord} <=> $b->{deepord}} map {($_,$_->descendants)} grep {$_} $bundle->attr("trees/S${lang}T")];
    }

    # target tectogrammatical trees
    my @tt_trees;
    foreach my $lang (@languages) {
        push @tt_trees, [sort {$a->{deepord} <=> $b->{deepord}} map {($_,$_->descendants)} grep {$_} $bundle->attr("trees/T${lang}T")];
    }

    # target analytical trees
    my @ta_trees;
    foreach my $lang (@languages) {
        push @ta_trees, [sort {$a->{ord} <=> $b->{ord}} map {($_,$_->descendants)} grep {$_} $bundle->attr("trees/T${lang}A")];
    }

    if (defined $bundle->{generic_subbundles}) {
        my @generic_subbundles = $bundle->{generic_subbundles}->elements;

        foreach my $element ( @generic_subbundles ) {
            my ( $name, $value ) = @$element;
            my $language = $value->{language};
            my $direction = $value->{direction};

            foreach my $tree_name (keys %{$value->{trees}}) {
                if ($tree_name =~ /^([a-z])/) {
                    my $layer = $1;
                    my $tree = $value->{trees}->{$tree_name};
                    my $array_name = lc($direction).lc($layer)."_trees";

                    my @nodes;
                    my $sort_attr_name;
                    if (defined $tree->{deepord}) {
                        $sort_attr_name = "deepord";
                    } elsif (defined $tree->{_ord}) {
                        $sort_attr_name = "_ord";
                    } else {
                        $sort_attr_name = "ord";
                    }

                    my @nodes = sort {$a->{$sort_attr_name} <=> $b->{$sort_attr_name}} ($tree, $tree->descendants);

                    if ($language eq "ar") {
                        @nodes = reverse @nodes;
                    }

                    eval "push \@$array_name, \\\@nodes";
                }
            }
            # print "value: $value\n";
            # if ( $value->{language} eq $language and $value->{direction} eq $direction ) {
            # }
        }
    }

    return (
        \@sm_trees,
        \@sn_trees,
        \@sp_trees,
        \@sa_trees,
        \@st_trees,
        \@tt_trees,
        \@ta_trees
    );
}

sub get_all_nodes_to_display {
    my $bundle = shift;

    my ($sm_trees,
        $sn_trees,
        $sp_trees,
        $sa_trees,
        $st_trees,
        $tt_trees,
        $ta_trees) = group_nodes_by_direction_and_layer($bundle);

    return (
        $bundle,
        _interleave(@$sm_trees),
        _interleave(@$sn_trees),
        _interleave(@$sp_trees),
        _interleave(@$sa_trees),
        _interleave(@$st_trees),
        _interleave(@$tt_trees),
        _interleave(@$ta_trees)
    );
}

sub _prepare_tree_names {
  #print "\n\n Executed file_opened_hook\n\n";

  # only apply this on TMT files
  return unless $grp->{FSFile};
  my $schema = PML::SchemaName();
  return unless defined $schema and $schema =~ /tmt_document/;

  foreach my $bundle ($grp->{FSFile}->trees) {
    foreach my $tree_name (keys %{$bundle->{trees}}) {
      #print "Processing tree $tree_name  \n";
      my $tree_root = $bundle->{trees}->{$tree_name};
      #print "TREE ROOT: $tree_root\n";
      foreach my $node ($tree_root, $tree_root->descendants) {
          $node->{_tree_name} = $tree_name;
      }
    }

    if (defined $bundle->{generic_subbundles}) {
      my @generic_subbundles = $bundle->{generic_subbundles}->elements;

      foreach my $element ( @generic_subbundles ) {
          my ( $name, $value ) = @$element;
          my $language = $value->{language};
          my $direction = $value->{direction};

          foreach my $tree_name (keys %{$value->{trees}}) {
              if ($tree_name =~ /^([a-z])/) {
                  my $layer = $1;
                  my $tree_root = $value->{trees}->{$tree_name};
                  my $tree_name = uc($direction).$language.uc($layer);
                  foreach my $node ($tree_root, $tree_root->descendants) {
                      $node->{_layer} = $layer;
                      $node->{_tree_name} = $tree_name;
                  }
              }
          }
      }
    }
  }
}

register_open_file_hook( \&_prepare_tree_names );

sub file_reloaded_hook {
  _prepare_tree_names();
}


my @morphcats = qw(pos subpos gender number case possgender possnumber
		   person tense grade negation voice reserve1 reserve2);

sub display_line {
  my ($node, $line) = @_;

  if (not $node->parent) {
    if ($line==1) {
      return "$node->{_tree_name}";
    }
  }

  else {
    if ($node->{_tree_name} eq "SEnglishP") {
      if ($line==1) {
        #	return "$node->{tag} $node->{phrase}";
        return ($node->{phrase} || $node->{form});
            } elsif ($line==2) {
				if ($node->{form}) {
				  return "$node->{form}";
                } elsif ($node->{is_head}) {
				  return "#{darkgrey}($node->{head_selection_rule})";
				}
            } elsif ($line==3) {
				if ($node->{form} and $node->{is_head}) {
				  return "#{darkgrey}($node->{head_selection_rule})";
				}
            }

        } elsif ($node->attr('id') =~ /SRussianA/) {
            if ($line == 1) {
	        return $node->attr('m/form');
            } elsif ($line == 2) {
	        return $node->attr('conll_deprel');
    }
        } elsif ($node->{_tree_name}=~/SCzechA$/) {
      if ($line==1) {
				return $node->attr('afun').($node->{is_member}?"_Co":"");
			} elsif ($line==2) {
				return $node->attr('m/form');
      }
        } elsif ($node->{_tree_name}=~/TCzechA$/) {
      if ($line==3) {
				return (join "",map {$node->attr("morphcat/$_")} @morphcats).($node->{is_member}?" Co":"");
            } elsif ($line==2) {
				return $node->attr('m/form');
            } elsif ($line==1) {
				return $node->attr('afun');
      }
        } elsif ($node->{_tree_name}=~/A$/ or $node->{_layer} eq "a") {
      if ($line==1) {
				return $node->attr('m/tag').($node->{is_member}?"_Co":"");
            } elsif ($line==2) {
				return $node->attr('m/form');
            } elsif ($line==3) {
				return $node->attr('afun');
      }
        } elsif ($node->{_tree_name}=~/M$/) {
      if ($line==1) {
				return "$node->{form}";
            } elsif ($line==2) {
				return "#{darkgreen}$node->{lemma}";
            } elsif ($line==3) {
				return "#{darkgreen}$node->{tag}";
      }
        } elsif ($node->{_tree_name}=~/TEnglishT/) {
      if ($line==1) {
				return "$node->{t_lemma}";
            } elsif ($line==2) {
				return "$node->{functor}".($node->{is_member}?"_CO":"");
            } elsif ($line==3) {
				my $color = ($node->{t_lemma} eq $node->{loglin_tlemma}) ? "#{blue}" : "#{red}";
				#return "${color}$node->{loglin_tlemma} / $node->{loglin_formeme}";
				return  "$node->{formeme}";
      }
        } elsif ($node->{_tree_name}=~/T(English|Czech)T/) {
            if ($line==1) {
				return "$node->{t_lemma} / $node->{formeme}";
            } elsif ($line==2) {
				my $color = ($node->{t_lemma} eq $node->{loglin_tlemma}) ? "#{blue}" : "#{red}";
				return "${color}$node->{loglin_tlemma} / $node->{loglin_formeme}";
            } elsif ($line==3) {
                return "$node->{functor}".($node->{is_member}?"_CO":"");
            }
        } elsif ($node->{_tree_name}=~/(English|Czech)T/ or $node->{_layer} eq "t") {
      if ($line==1) {
				return "$node->{t_lemma}";
      } elsif ($line==2) {
				return "$node->{functor}".($node->{is_member}?"_CO":"");
      } elsif ($line==3) {
				return "#{red}$node->{formeme}"
      }
        } elsif ($node->{_tree_name}=~/SCzechN/) {
      if ($line==1) {
        return $node->attr('ne_type');
      } elsif ($line==2) {
        return $node->attr('normalized_name');
      }
    } else {
      return "???";
    }
  }
}

my %treename2nodecolor = (
			  'SEnglishM' => 'grey',
			  'SEnglishP' => 'blue',
			  'SEnglishA' => '#10AA00',
			  'SEnglishT' => '#FF5555',
			  'SCzechT' => '#ff9999',
        'SCzechA' => '#10AA00',
			  'TEnglishT' => '#FF0000',
			  'TEnglishA' => '#10AA00',
			 );

my %layername2nodecolor = (
    'm' => 'grey',
    'p' => 'blue',
    'a' => '#10AA00',
    't' => '#FF5555',
);


sub get_root_style() {

    return "#{nodeXSkip:15} #{nodeYSkip:2} #{lineSpacing:0.7} #{BaseXPos:0} #{BaseYPos:10} #{BalanceTree:1} #{skipHiddenLevels:0}";

}

sub get_node_style($) {
  my ($node) = @_;

  # ---- node color
  my $node_color;
  $node_color;
  if ($node->{id}=~/TEnglishT/ and $node->{is_clause_head}) {
  	$node_color = 'yellow';
    } else {
  	$node_color = $treename2nodecolor{$node->{_tree_name}} || $layername2nodecolor{$node->{_layer}};
  }

  # ---- line color
  my $line_color;
  if ($node->{id}=~/SEnglishP/ and not $node->{is_head}) { # hrany k nehlavovym potomkum jsou svetlejsi
    $line_color = "#33ccff";
    } elsif ($node->{id}=~/S(Czech|English)T/ and $node->attr('align/is_aligned_edge')) {
    $line_color="darkred";
    } elsif ($node->{id}=~/(Czech|English)A/ and # hrany mezi uzly, ktere nebudou slouceny v jeden t-uzel, jsou svetlejsi
                 not ($node->{is_aux_to_parent} or $node->{parent_is_aux} or ($node->parent and $node->parent()->{is_aux_to_child}) or ($node->{edge_to_collapse}))) {
    $line_color = "#55FF55";
    } else {
        $line_color = $treename2nodecolor{$node->{_tree_name}} || $layername2nodecolor{$node->{_layer}};
  }
    ;

  # zmena barvy uzlu
  if ($node->{id}=~/(Czech|English)T/ and $node->{nodetype} ne "complex") {
    $node_color = "grey";
  }



  # ---- line width # !!! tohle by chtelo napsat prehledneji
  my $line_width;
  my $id = $node->{id};
  if ($node->{id}=~/SEnglishP/ and $node->{is_head}) { # hrany k hlavovym potomkum jsou tlustejsi
    $line_width = 5;
    } elsif ($node->{id}=~/(Czech|English)A/ and # hrany mezi uzly, ktere budou slouceny v jeden t-uzel, jsou tlustejsi
                 ($node->{is_aux_to_parent} or $node->{parent_is_aux} or ($node->parent and $node->parent()->{is_aux_to_child}) or ($node->{edge_to_collapse}))) {
    $line_width = 5;
    } elsif ($id=~/(Czech|English)T/ and $node->attr('align/is_aligned_edge')) { # paralelni zavislosti (spojujici stejne uzly) jsou zvyraznene
    $line_width = 5;
    } else {
    $line_width = 2;
    }
    ;

  # --- node size
#  my $node_size;
#  if ($node->{nodetype} eq "complex") {
#    $node_size=" #{Node-height:8} #{Node-width:8} ";
#  }

  return "#{Line-width:$line_width} #{Oval-fill:$node_color} #{Line-fill:$line_color}".
  "#{Oval-width:1}#{CurrentOval-width:4}#{CurrentOval-outline:red}";
}

sub get_generic_subbundle {
    my ( $fs_bundle_root, $language, $direction ) = @_;
    if ( defined $fs_bundle_root->{generic_subbundles} ) {
        foreach my $element ( $fs_bundle_root->{generic_subbundles}->elements ) {
            my ( $name, $value ) = @$element;
            if ( $value->{language} eq $language and $value->{direction} eq $direction ) {
                return $value;
            }
        }
    }
    return undef;
}
